# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class ManualJournal(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "narration": "str",
        "journal_lines": "list[ManualJournalLine]",
        "date": "date[ms-format]",
        "line_amount_types": "LineAmountTypes",
        "status": "str",
        "url": "str",
        "show_on_cash_basis_reports": "bool",
        "has_attachments": "bool",
        "updated_date_utc": "datetime[ms-format]",
        "manual_journal_id": "str",
        "status_attribute_string": "str",
        "warnings": "list[ValidationError]",
        "validation_errors": "list[ValidationError]",
        "attachments": "list[Attachment]",
    }

    attribute_map = {
        "narration": "Narration",
        "journal_lines": "JournalLines",
        "date": "Date",
        "line_amount_types": "LineAmountTypes",
        "status": "Status",
        "url": "Url",
        "show_on_cash_basis_reports": "ShowOnCashBasisReports",
        "has_attachments": "HasAttachments",
        "updated_date_utc": "UpdatedDateUTC",
        "manual_journal_id": "ManualJournalID",
        "status_attribute_string": "StatusAttributeString",
        "warnings": "Warnings",
        "validation_errors": "ValidationErrors",
        "attachments": "Attachments",
    }

    def __init__(
        self,
        narration=None,
        journal_lines=None,
        date=None,
        line_amount_types=None,
        status=None,
        url=None,
        show_on_cash_basis_reports=None,
        has_attachments=False,
        updated_date_utc=None,
        manual_journal_id=None,
        status_attribute_string=None,
        warnings=None,
        validation_errors=None,
        attachments=None,
    ):  # noqa: E501
        """ManualJournal - a model defined in OpenAPI"""  # noqa: E501

        self._narration = None
        self._journal_lines = None
        self._date = None
        self._line_amount_types = None
        self._status = None
        self._url = None
        self._show_on_cash_basis_reports = None
        self._has_attachments = None
        self._updated_date_utc = None
        self._manual_journal_id = None
        self._status_attribute_string = None
        self._warnings = None
        self._validation_errors = None
        self._attachments = None
        self.discriminator = None

        self.narration = narration
        if journal_lines is not None:
            self.journal_lines = journal_lines
        if date is not None:
            self.date = date
        if line_amount_types is not None:
            self.line_amount_types = line_amount_types
        if status is not None:
            self.status = status
        if url is not None:
            self.url = url
        if show_on_cash_basis_reports is not None:
            self.show_on_cash_basis_reports = show_on_cash_basis_reports
        if has_attachments is not None:
            self.has_attachments = has_attachments
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc
        if manual_journal_id is not None:
            self.manual_journal_id = manual_journal_id
        if status_attribute_string is not None:
            self.status_attribute_string = status_attribute_string
        if warnings is not None:
            self.warnings = warnings
        if validation_errors is not None:
            self.validation_errors = validation_errors
        if attachments is not None:
            self.attachments = attachments

    @property
    def narration(self):
        """Gets the narration of this ManualJournal.  # noqa: E501

        Description of journal being posted  # noqa: E501

        :return: The narration of this ManualJournal.  # noqa: E501
        :rtype: str
        """
        return self._narration

    @narration.setter
    def narration(self, narration):
        """Sets the narration of this ManualJournal.

        Description of journal being posted  # noqa: E501

        :param narration: The narration of this ManualJournal.  # noqa: E501
        :type: str
        """
        if narration is None:
            raise ValueError(
                "Invalid value for `narration`, must not be `None`"
            )  # noqa: E501

        self._narration = narration

    @property
    def journal_lines(self):
        """Gets the journal_lines of this ManualJournal.  # noqa: E501

        See JournalLines  # noqa: E501

        :return: The journal_lines of this ManualJournal.  # noqa: E501
        :rtype: list[ManualJournalLine]
        """
        return self._journal_lines

    @journal_lines.setter
    def journal_lines(self, journal_lines):
        """Sets the journal_lines of this ManualJournal.

        See JournalLines  # noqa: E501

        :param journal_lines: The journal_lines of this ManualJournal.  # noqa: E501
        :type: list[ManualJournalLine]
        """

        self._journal_lines = journal_lines

    @property
    def date(self):
        """Gets the date of this ManualJournal.  # noqa: E501

        Date journal was posted – YYYY-MM-DD  # noqa: E501

        :return: The date of this ManualJournal.  # noqa: E501
        :rtype: date
        """
        return self._date

    @date.setter
    def date(self, date):
        """Sets the date of this ManualJournal.

        Date journal was posted – YYYY-MM-DD  # noqa: E501

        :param date: The date of this ManualJournal.  # noqa: E501
        :type: date
        """

        self._date = date

    @property
    def line_amount_types(self):
        """Gets the line_amount_types of this ManualJournal.  # noqa: E501


        :return: The line_amount_types of this ManualJournal.  # noqa: E501
        :rtype: LineAmountTypes
        """
        return self._line_amount_types

    @line_amount_types.setter
    def line_amount_types(self, line_amount_types):
        """Sets the line_amount_types of this ManualJournal.


        :param line_amount_types: The line_amount_types of this ManualJournal.  # noqa: E501
        :type: LineAmountTypes
        """

        self._line_amount_types = line_amount_types

    @property
    def status(self):
        """Gets the status of this ManualJournal.  # noqa: E501

        See Manual Journal Status Codes  # noqa: E501

        :return: The status of this ManualJournal.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ManualJournal.

        See Manual Journal Status Codes  # noqa: E501

        :param status: The status of this ManualJournal.  # noqa: E501
        :type: str
        """
        allowed_values = [
            "DRAFT",
            "POSTED",
            "DELETED",
            "VOIDED",
            "ARCHIVED",
            "None",
        ]  # noqa: E501

        if status:
            if status not in allowed_values:
                raise ValueError(
                    "Invalid value for `status` ({0}), must be one of {1}".format(  # noqa: E501
                        status, allowed_values
                    )
                )

        self._status = status

    @property
    def url(self):
        """Gets the url of this ManualJournal.  # noqa: E501

        Url link to a source document – shown as “Go to [appName]” in the Xero app  # noqa: E501

        :return: The url of this ManualJournal.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this ManualJournal.

        Url link to a source document – shown as “Go to [appName]” in the Xero app  # noqa: E501

        :param url: The url of this ManualJournal.  # noqa: E501
        :type: str
        """

        self._url = url

    @property
    def show_on_cash_basis_reports(self):
        """Gets the show_on_cash_basis_reports of this ManualJournal.  # noqa: E501

        Boolean – default is true if not specified  # noqa: E501

        :return: The show_on_cash_basis_reports of this ManualJournal.  # noqa: E501
        :rtype: bool
        """
        return self._show_on_cash_basis_reports

    @show_on_cash_basis_reports.setter
    def show_on_cash_basis_reports(self, show_on_cash_basis_reports):
        """Sets the show_on_cash_basis_reports of this ManualJournal.

        Boolean – default is true if not specified  # noqa: E501

        :param show_on_cash_basis_reports: The show_on_cash_basis_reports of this ManualJournal.  # noqa: E501
        :type: bool
        """

        self._show_on_cash_basis_reports = show_on_cash_basis_reports

    @property
    def has_attachments(self):
        """Gets the has_attachments of this ManualJournal.  # noqa: E501

        Boolean to indicate if a manual journal has an attachment  # noqa: E501

        :return: The has_attachments of this ManualJournal.  # noqa: E501
        :rtype: bool
        """
        return self._has_attachments

    @has_attachments.setter
    def has_attachments(self, has_attachments):
        """Sets the has_attachments of this ManualJournal.

        Boolean to indicate if a manual journal has an attachment  # noqa: E501

        :param has_attachments: The has_attachments of this ManualJournal.  # noqa: E501
        :type: bool
        """

        self._has_attachments = has_attachments

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this ManualJournal.  # noqa: E501

        Last modified date UTC format  # noqa: E501

        :return: The updated_date_utc of this ManualJournal.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this ManualJournal.

        Last modified date UTC format  # noqa: E501

        :param updated_date_utc: The updated_date_utc of this ManualJournal.  # noqa: E501
        :type: datetime
        """

        self._updated_date_utc = updated_date_utc

    @property
    def manual_journal_id(self):
        """Gets the manual_journal_id of this ManualJournal.  # noqa: E501

        The Xero identifier for a Manual Journal  # noqa: E501

        :return: The manual_journal_id of this ManualJournal.  # noqa: E501
        :rtype: str
        """
        return self._manual_journal_id

    @manual_journal_id.setter
    def manual_journal_id(self, manual_journal_id):
        """Sets the manual_journal_id of this ManualJournal.

        The Xero identifier for a Manual Journal  # noqa: E501

        :param manual_journal_id: The manual_journal_id of this ManualJournal.  # noqa: E501
        :type: str
        """

        self._manual_journal_id = manual_journal_id

    @property
    def status_attribute_string(self):
        """Gets the status_attribute_string of this ManualJournal.  # noqa: E501

        A string to indicate if a invoice status  # noqa: E501

        :return: The status_attribute_string of this ManualJournal.  # noqa: E501
        :rtype: str
        """
        return self._status_attribute_string

    @status_attribute_string.setter
    def status_attribute_string(self, status_attribute_string):
        """Sets the status_attribute_string of this ManualJournal.

        A string to indicate if a invoice status  # noqa: E501

        :param status_attribute_string: The status_attribute_string of this ManualJournal.  # noqa: E501
        :type: str
        """

        self._status_attribute_string = status_attribute_string

    @property
    def warnings(self):
        """Gets the warnings of this ManualJournal.  # noqa: E501

        Displays array of warning messages from the API  # noqa: E501

        :return: The warnings of this ManualJournal.  # noqa: E501
        :rtype: list[ValidationError]
        """
        return self._warnings

    @warnings.setter
    def warnings(self, warnings):
        """Sets the warnings of this ManualJournal.

        Displays array of warning messages from the API  # noqa: E501

        :param warnings: The warnings of this ManualJournal.  # noqa: E501
        :type: list[ValidationError]
        """

        self._warnings = warnings

    @property
    def validation_errors(self):
        """Gets the validation_errors of this ManualJournal.  # noqa: E501

        Displays array of validation error messages from the API  # noqa: E501

        :return: The validation_errors of this ManualJournal.  # noqa: E501
        :rtype: list[ValidationError]
        """
        return self._validation_errors

    @validation_errors.setter
    def validation_errors(self, validation_errors):
        """Sets the validation_errors of this ManualJournal.

        Displays array of validation error messages from the API  # noqa: E501

        :param validation_errors: The validation_errors of this ManualJournal.  # noqa: E501
        :type: list[ValidationError]
        """

        self._validation_errors = validation_errors

    @property
    def attachments(self):
        """Gets the attachments of this ManualJournal.  # noqa: E501

        Displays array of attachments from the API  # noqa: E501

        :return: The attachments of this ManualJournal.  # noqa: E501
        :rtype: list[Attachment]
        """
        return self._attachments

    @attachments.setter
    def attachments(self, attachments):
        """Sets the attachments of this ManualJournal.

        Displays array of attachments from the API  # noqa: E501

        :param attachments: The attachments of this ManualJournal.  # noqa: E501
        :type: list[Attachment]
        """

        self._attachments = attachments



# coding: utf-8

"""
    Xero Assets API

    The Assets API exposes fixed asset related functions of the Xero Accounting application and can be used for a variety of purposes such as creating assets, retrieving asset valuations etc.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Pagination(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "page": "int",
        "page_size": "int",
        "page_count": "int",
        "item_count": "int",
    }

    attribute_map = {
        "page": "page",
        "page_size": "pageSize",
        "page_count": "pageCount",
        "item_count": "itemCount",
    }

    def __init__(
        self, page=None, page_size=None, page_count=None, item_count=None
    ):  # noqa: E501
        """Pagination - a model defined in OpenAPI"""  # noqa: E501

        self._page = None
        self._page_size = None
        self._page_count = None
        self._item_count = None
        self.discriminator = None

        if page is not None:
            self.page = page
        if page_size is not None:
            self.page_size = page_size
        if page_count is not None:
            self.page_count = page_count
        if item_count is not None:
            self.item_count = item_count

    @property
    def page(self):
        """Gets the page of this Pagination.  # noqa: E501


        :return: The page of this Pagination.  # noqa: E501
        :rtype: int
        """
        return self._page

    @page.setter
    def page(self, page):
        """Sets the page of this Pagination.


        :param page: The page of this Pagination.  # noqa: E501
        :type: int
        """

        self._page = page

    @property
    def page_size(self):
        """Gets the page_size of this Pagination.  # noqa: E501


        :return: The page_size of this Pagination.  # noqa: E501
        :rtype: int
        """
        return self._page_size

    @page_size.setter
    def page_size(self, page_size):
        """Sets the page_size of this Pagination.


        :param page_size: The page_size of this Pagination.  # noqa: E501
        :type: int
        """

        self._page_size = page_size

    @property
    def page_count(self):
        """Gets the page_count of this Pagination.  # noqa: E501


        :return: The page_count of this Pagination.  # noqa: E501
        :rtype: int
        """
        return self._page_count

    @page_count.setter
    def page_count(self, page_count):
        """Sets the page_count of this Pagination.


        :param page_count: The page_count of this Pagination.  # noqa: E501
        :type: int
        """

        self._page_count = page_count

    @property
    def item_count(self):
        """Gets the item_count of this Pagination.  # noqa: E501


        :return: The item_count of this Pagination.  # noqa: E501
        :rtype: int
        """
        return self._item_count

    @item_count.setter
    def item_count(self, item_count):
        """Sets the item_count of this Pagination.


        :param item_count: The item_count of this Pagination.  # noqa: E501
        :type: int
        """

        self._item_count = item_count



# coding: utf-8

"""
    Xero Payroll NZ

    This is the Xero Payroll API for orgs in the NZ region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class BankAccount(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "account_name": "str",
        "account_number": "str",
        "sort_code": "str",
        "particulars": "str",
        "code": "str",
        "dollar_amount": "float",
        "reference": "str",
        "calculation_type": "str",
    }

    attribute_map = {
        "account_name": "accountName",
        "account_number": "accountNumber",
        "sort_code": "sortCode",
        "particulars": "particulars",
        "code": "code",
        "dollar_amount": "dollarAmount",
        "reference": "reference",
        "calculation_type": "calculationType",
    }

    def __init__(
        self,
        account_name=None,
        account_number=None,
        sort_code=None,
        particulars=None,
        code=None,
        dollar_amount=None,
        reference=None,
        calculation_type=None,
    ):  # noqa: E501
        """BankAccount - a model defined in OpenAPI"""  # noqa: E501

        self._account_name = None
        self._account_number = None
        self._sort_code = None
        self._particulars = None
        self._code = None
        self._dollar_amount = None
        self._reference = None
        self._calculation_type = None
        self.discriminator = None

        self.account_name = account_name
        self.account_number = account_number
        self.sort_code = sort_code
        if particulars is not None:
            self.particulars = particulars
        if code is not None:
            self.code = code
        if dollar_amount is not None:
            self.dollar_amount = dollar_amount
        if reference is not None:
            self.reference = reference
        if calculation_type is not None:
            self.calculation_type = calculation_type

    @property
    def account_name(self):
        """Gets the account_name of this BankAccount.  # noqa: E501

        Bank account name (max length = 32)  # noqa: E501

        :return: The account_name of this BankAccount.  # noqa: E501
        :rtype: str
        """
        return self._account_name

    @account_name.setter
    def account_name(self, account_name):
        """Sets the account_name of this BankAccount.

        Bank account name (max length = 32)  # noqa: E501

        :param account_name: The account_name of this BankAccount.  # noqa: E501
        :type: str
        """
        if account_name is None:
            raise ValueError(
                "Invalid value for `account_name`, must not be `None`"
            )  # noqa: E501

        self._account_name = account_name

    @property
    def account_number(self):
        """Gets the account_number of this BankAccount.  # noqa: E501

        Bank account number (digits only; max length = 8)  # noqa: E501

        :return: The account_number of this BankAccount.  # noqa: E501
        :rtype: str
        """
        return self._account_number

    @account_number.setter
    def account_number(self, account_number):
        """Sets the account_number of this BankAccount.

        Bank account number (digits only; max length = 8)  # noqa: E501

        :param account_number: The account_number of this BankAccount.  # noqa: E501
        :type: str
        """
        if account_number is None:
            raise ValueError(
                "Invalid value for `account_number`, must not be `None`"
            )  # noqa: E501

        self._account_number = account_number

    @property
    def sort_code(self):
        """Gets the sort_code of this BankAccount.  # noqa: E501

        Bank account sort code (6 digits)  # noqa: E501

        :return: The sort_code of this BankAccount.  # noqa: E501
        :rtype: str
        """
        return self._sort_code

    @sort_code.setter
    def sort_code(self, sort_code):
        """Sets the sort_code of this BankAccount.

        Bank account sort code (6 digits)  # noqa: E501

        :param sort_code: The sort_code of this BankAccount.  # noqa: E501
        :type: str
        """
        if sort_code is None:
            raise ValueError(
                "Invalid value for `sort_code`, must not be `None`"
            )  # noqa: E501

        self._sort_code = sort_code

    @property
    def particulars(self):
        """Gets the particulars of this BankAccount.  # noqa: E501

        Particulars that appear on the statement.  # noqa: E501

        :return: The particulars of this BankAccount.  # noqa: E501
        :rtype: str
        """
        return self._particulars

    @particulars.setter
    def particulars(self, particulars):
        """Sets the particulars of this BankAccount.

        Particulars that appear on the statement.  # noqa: E501

        :param particulars: The particulars of this BankAccount.  # noqa: E501
        :type: str
        """

        self._particulars = particulars

    @property
    def code(self):
        """Gets the code of this BankAccount.  # noqa: E501

        Code of a transaction that appear on the statement.  # noqa: E501

        :return: The code of this BankAccount.  # noqa: E501
        :rtype: str
        """
        return self._code

    @code.setter
    def code(self, code):
        """Sets the code of this BankAccount.

        Code of a transaction that appear on the statement.  # noqa: E501

        :param code: The code of this BankAccount.  # noqa: E501
        :type: str
        """

        self._code = code

    @property
    def dollar_amount(self):
        """Gets the dollar_amount of this BankAccount.  # noqa: E501

        Dollar amount of a transaction.  # noqa: E501

        :return: The dollar_amount of this BankAccount.  # noqa: E501
        :rtype: float
        """
        return self._dollar_amount

    @dollar_amount.setter
    def dollar_amount(self, dollar_amount):
        """Sets the dollar_amount of this BankAccount.

        Dollar amount of a transaction.  # noqa: E501

        :param dollar_amount: The dollar_amount of this BankAccount.  # noqa: E501
        :type: float
        """

        self._dollar_amount = dollar_amount

    @property
    def reference(self):
        """Gets the reference of this BankAccount.  # noqa: E501

        Statement Text/reference for a transaction that appear on the statement.  # noqa: E501

        :return: The reference of this BankAccount.  # noqa: E501
        :rtype: str
        """
        return self._reference

    @reference.setter
    def reference(self, reference):
        """Sets the reference of this BankAccount.

        Statement Text/reference for a transaction that appear on the statement.  # noqa: E501

        :param reference: The reference of this BankAccount.  # noqa: E501
        :type: str
        """

        self._reference = reference

    @property
    def calculation_type(self):
        """Gets the calculation_type of this BankAccount.  # noqa: E501

        Calculation type for the transaction can be 'Fixed Amount' or 'Balance'  # noqa: E501

        :return: The calculation_type of this BankAccount.  # noqa: E501
        :rtype: str
        """
        return self._calculation_type

    @calculation_type.setter
    def calculation_type(self, calculation_type):
        """Sets the calculation_type of this BankAccount.

        Calculation type for the transaction can be 'Fixed Amount' or 'Balance'  # noqa: E501

        :param calculation_type: The calculation_type of this BankAccount.  # noqa: E501
        :type: str
        """
        allowed_values = ["FixedAmount", "Balance", "None"]  # noqa: E501

        if calculation_type:
            if calculation_type not in allowed_values:
                raise ValueError(
                    "Invalid value for `calculation_type` ({0}), must be one of {1}".format(  # noqa: E501
                        calculation_type, allowed_values
                    )
                )

        self._calculation_type = calculation_type



# coding: utf-8

"""
    Xero Payroll UK

    This is the Xero Payroll API for orgs in the UK region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class EarningsTemplate(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "pay_template_earning_id": "str",
        "rate_per_unit": "float",
        "number_of_units": "float",
        "fixed_amount": "float",
        "earnings_rate_id": "str",
        "name": "str",
    }

    attribute_map = {
        "pay_template_earning_id": "payTemplateEarningID",
        "rate_per_unit": "ratePerUnit",
        "number_of_units": "numberOfUnits",
        "fixed_amount": "fixedAmount",
        "earnings_rate_id": "earningsRateID",
        "name": "name",
    }

    def __init__(
        self,
        pay_template_earning_id=None,
        rate_per_unit=None,
        number_of_units=None,
        fixed_amount=None,
        earnings_rate_id=None,
        name=None,
    ):  # noqa: E501
        """EarningsTemplate - a model defined in OpenAPI"""  # noqa: E501

        self._pay_template_earning_id = None
        self._rate_per_unit = None
        self._number_of_units = None
        self._fixed_amount = None
        self._earnings_rate_id = None
        self._name = None
        self.discriminator = None

        if pay_template_earning_id is not None:
            self.pay_template_earning_id = pay_template_earning_id
        if rate_per_unit is not None:
            self.rate_per_unit = rate_per_unit
        if number_of_units is not None:
            self.number_of_units = number_of_units
        if fixed_amount is not None:
            self.fixed_amount = fixed_amount
        if earnings_rate_id is not None:
            self.earnings_rate_id = earnings_rate_id
        if name is not None:
            self.name = name

    @property
    def pay_template_earning_id(self):
        """Gets the pay_template_earning_id of this EarningsTemplate.  # noqa: E501

        The Xero identifier for the earnings template  # noqa: E501

        :return: The pay_template_earning_id of this EarningsTemplate.  # noqa: E501
        :rtype: str
        """
        return self._pay_template_earning_id

    @pay_template_earning_id.setter
    def pay_template_earning_id(self, pay_template_earning_id):
        """Sets the pay_template_earning_id of this EarningsTemplate.

        The Xero identifier for the earnings template  # noqa: E501

        :param pay_template_earning_id: The pay_template_earning_id of this EarningsTemplate.  # noqa: E501
        :type: str
        """

        self._pay_template_earning_id = pay_template_earning_id

    @property
    def rate_per_unit(self):
        """Gets the rate_per_unit of this EarningsTemplate.  # noqa: E501

        The rate per unit  # noqa: E501

        :return: The rate_per_unit of this EarningsTemplate.  # noqa: E501
        :rtype: float
        """
        return self._rate_per_unit

    @rate_per_unit.setter
    def rate_per_unit(self, rate_per_unit):
        """Sets the rate_per_unit of this EarningsTemplate.

        The rate per unit  # noqa: E501

        :param rate_per_unit: The rate_per_unit of this EarningsTemplate.  # noqa: E501
        :type: float
        """

        self._rate_per_unit = rate_per_unit

    @property
    def number_of_units(self):
        """Gets the number_of_units of this EarningsTemplate.  # noqa: E501

        The rate per unit  # noqa: E501

        :return: The number_of_units of this EarningsTemplate.  # noqa: E501
        :rtype: float
        """
        return self._number_of_units

    @number_of_units.setter
    def number_of_units(self, number_of_units):
        """Sets the number_of_units of this EarningsTemplate.

        The rate per unit  # noqa: E501

        :param number_of_units: The number_of_units of this EarningsTemplate.  # noqa: E501
        :type: float
        """

        self._number_of_units = number_of_units

    @property
    def fixed_amount(self):
        """Gets the fixed_amount of this EarningsTemplate.  # noqa: E501

        The fixed amount per period  # noqa: E501

        :return: The fixed_amount of this EarningsTemplate.  # noqa: E501
        :rtype: float
        """
        return self._fixed_amount

    @fixed_amount.setter
    def fixed_amount(self, fixed_amount):
        """Sets the fixed_amount of this EarningsTemplate.

        The fixed amount per period  # noqa: E501

        :param fixed_amount: The fixed_amount of this EarningsTemplate.  # noqa: E501
        :type: float
        """

        self._fixed_amount = fixed_amount

    @property
    def earnings_rate_id(self):
        """Gets the earnings_rate_id of this EarningsTemplate.  # noqa: E501

        The corresponding earnings rate identifier  # noqa: E501

        :return: The earnings_rate_id of this EarningsTemplate.  # noqa: E501
        :rtype: str
        """
        return self._earnings_rate_id

    @earnings_rate_id.setter
    def earnings_rate_id(self, earnings_rate_id):
        """Sets the earnings_rate_id of this EarningsTemplate.

        The corresponding earnings rate identifier  # noqa: E501

        :param earnings_rate_id: The earnings_rate_id of this EarningsTemplate.  # noqa: E501
        :type: str
        """

        self._earnings_rate_id = earnings_rate_id

    @property
    def name(self):
        """Gets the name of this EarningsTemplate.  # noqa: E501

        The read-only name of the Earning Template.  # noqa: E501

        :return: The name of this EarningsTemplate.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this EarningsTemplate.

        The read-only name of the Earning Template.  # noqa: E501

        :param name: The name of this EarningsTemplate.  # noqa: E501
        :type: str
        """

        self._name = name



# coding: utf-8

"""
    Xero Payroll UK

    This is the Xero Payroll API for orgs in the UK region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class EmployeeOpeningBalances(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "statutory_adoption_pay": "float",
        "statutory_maternity_pay": "float",
        "statutory_paternity_pay": "float",
        "statutory_shared_parental_pay": "float",
        "statutory_sick_pay": "float",
        "prior_employee_number": "float",
    }

    attribute_map = {
        "statutory_adoption_pay": "statutoryAdoptionPay",
        "statutory_maternity_pay": "statutoryMaternityPay",
        "statutory_paternity_pay": "statutoryPaternityPay",
        "statutory_shared_parental_pay": "statutorySharedParentalPay",
        "statutory_sick_pay": "statutorySickPay",
        "prior_employee_number": "priorEmployeeNumber",
    }

    def __init__(
        self,
        statutory_adoption_pay=None,
        statutory_maternity_pay=None,
        statutory_paternity_pay=None,
        statutory_shared_parental_pay=None,
        statutory_sick_pay=None,
        prior_employee_number=None,
    ):  # noqa: E501
        """EmployeeOpeningBalances - a model defined in OpenAPI"""  # noqa: E501

        self._statutory_adoption_pay = None
        self._statutory_maternity_pay = None
        self._statutory_paternity_pay = None
        self._statutory_shared_parental_pay = None
        self._statutory_sick_pay = None
        self._prior_employee_number = None
        self.discriminator = None

        if statutory_adoption_pay is not None:
            self.statutory_adoption_pay = statutory_adoption_pay
        if statutory_maternity_pay is not None:
            self.statutory_maternity_pay = statutory_maternity_pay
        if statutory_paternity_pay is not None:
            self.statutory_paternity_pay = statutory_paternity_pay
        if statutory_shared_parental_pay is not None:
            self.statutory_shared_parental_pay = statutory_shared_parental_pay
        if statutory_sick_pay is not None:
            self.statutory_sick_pay = statutory_sick_pay
        if prior_employee_number is not None:
            self.prior_employee_number = prior_employee_number

    @property
    def statutory_adoption_pay(self):
        """Gets the statutory_adoption_pay of this EmployeeOpeningBalances.  # noqa: E501

        The total accumulated statutory adoption pay amount received by the employee for current fiscal year to date  # noqa: E501

        :return: The statutory_adoption_pay of this EmployeeOpeningBalances.  # noqa: E501
        :rtype: float
        """
        return self._statutory_adoption_pay

    @statutory_adoption_pay.setter
    def statutory_adoption_pay(self, statutory_adoption_pay):
        """Sets the statutory_adoption_pay of this EmployeeOpeningBalances.

        The total accumulated statutory adoption pay amount received by the employee for current fiscal year to date  # noqa: E501

        :param statutory_adoption_pay: The statutory_adoption_pay of this EmployeeOpeningBalances.  # noqa: E501
        :type: float
        """

        self._statutory_adoption_pay = statutory_adoption_pay

    @property
    def statutory_maternity_pay(self):
        """Gets the statutory_maternity_pay of this EmployeeOpeningBalances.  # noqa: E501

        The total accumulated statutory maternity pay amount received by the employee for current fiscal year to date  # noqa: E501

        :return: The statutory_maternity_pay of this EmployeeOpeningBalances.  # noqa: E501
        :rtype: float
        """
        return self._statutory_maternity_pay

    @statutory_maternity_pay.setter
    def statutory_maternity_pay(self, statutory_maternity_pay):
        """Sets the statutory_maternity_pay of this EmployeeOpeningBalances.

        The total accumulated statutory maternity pay amount received by the employee for current fiscal year to date  # noqa: E501

        :param statutory_maternity_pay: The statutory_maternity_pay of this EmployeeOpeningBalances.  # noqa: E501
        :type: float
        """

        self._statutory_maternity_pay = statutory_maternity_pay

    @property
    def statutory_paternity_pay(self):
        """Gets the statutory_paternity_pay of this EmployeeOpeningBalances.  # noqa: E501

        The total accumulated statutory paternity pay amount received by the employee for current fiscal year to date  # noqa: E501

        :return: The statutory_paternity_pay of this EmployeeOpeningBalances.  # noqa: E501
        :rtype: float
        """
        return self._statutory_paternity_pay

    @statutory_paternity_pay.setter
    def statutory_paternity_pay(self, statutory_paternity_pay):
        """Sets the statutory_paternity_pay of this EmployeeOpeningBalances.

        The total accumulated statutory paternity pay amount received by the employee for current fiscal year to date  # noqa: E501

        :param statutory_paternity_pay: The statutory_paternity_pay of this EmployeeOpeningBalances.  # noqa: E501
        :type: float
        """

        self._statutory_paternity_pay = statutory_paternity_pay

    @property
    def statutory_shared_parental_pay(self):
        """Gets the statutory_shared_parental_pay of this EmployeeOpeningBalances.  # noqa: E501

        The total accumulated statutory shared parental pay amount received by the employee for current fiscal year to date  # noqa: E501

        :return: The statutory_shared_parental_pay of this EmployeeOpeningBalances.  # noqa: E501
        :rtype: float
        """
        return self._statutory_shared_parental_pay

    @statutory_shared_parental_pay.setter
    def statutory_shared_parental_pay(self, statutory_shared_parental_pay):
        """Sets the statutory_shared_parental_pay of this EmployeeOpeningBalances.

        The total accumulated statutory shared parental pay amount received by the employee for current fiscal year to date  # noqa: E501

        :param statutory_shared_parental_pay: The statutory_shared_parental_pay of this EmployeeOpeningBalances.  # noqa: E501
        :type: float
        """

        self._statutory_shared_parental_pay = statutory_shared_parental_pay

    @property
    def statutory_sick_pay(self):
        """Gets the statutory_sick_pay of this EmployeeOpeningBalances.  # noqa: E501

        The total accumulated statutory sick pay amount received by the employee for current fiscal year to date  # noqa: E501

        :return: The statutory_sick_pay of this EmployeeOpeningBalances.  # noqa: E501
        :rtype: float
        """
        return self._statutory_sick_pay

    @statutory_sick_pay.setter
    def statutory_sick_pay(self, statutory_sick_pay):
        """Sets the statutory_sick_pay of this EmployeeOpeningBalances.

        The total accumulated statutory sick pay amount received by the employee for current fiscal year to date  # noqa: E501

        :param statutory_sick_pay: The statutory_sick_pay of this EmployeeOpeningBalances.  # noqa: E501
        :type: float
        """

        self._statutory_sick_pay = statutory_sick_pay

    @property
    def prior_employee_number(self):
        """Gets the prior_employee_number of this EmployeeOpeningBalances.  # noqa: E501

        The unique employee number issued by the employee's former employer  # noqa: E501

        :return: The prior_employee_number of this EmployeeOpeningBalances.  # noqa: E501
        :rtype: float
        """
        return self._prior_employee_number

    @prior_employee_number.setter
    def prior_employee_number(self, prior_employee_number):
        """Sets the prior_employee_number of this EmployeeOpeningBalances.

        The unique employee number issued by the employee's former employer  # noqa: E501

        :param prior_employee_number: The prior_employee_number of this EmployeeOpeningBalances.  # noqa: E501
        :type: float
        """

        self._prior_employee_number = prior_employee_number



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Account(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "account_id": "str",
        "type": "AccountType",
        "code": "str",
        "name": "str",
    }

    attribute_map = {
        "account_id": "AccountID",
        "type": "Type",
        "code": "Code",
        "name": "Name",
    }

    def __init__(self, account_id=None, type=None, code=None, name=None):  # noqa: E501
        """Account - a model defined in OpenAPI"""  # noqa: E501

        self._account_id = None
        self._type = None
        self._code = None
        self._name = None
        self.discriminator = None

        if account_id is not None:
            self.account_id = account_id
        if type is not None:
            self.type = type
        if code is not None:
            self.code = code
        if name is not None:
            self.name = name

    @property
    def account_id(self):
        """Gets the account_id of this Account.  # noqa: E501

        Xero identifier for accounts  # noqa: E501

        :return: The account_id of this Account.  # noqa: E501
        :rtype: str
        """
        return self._account_id

    @account_id.setter
    def account_id(self, account_id):
        """Sets the account_id of this Account.

        Xero identifier for accounts  # noqa: E501

        :param account_id: The account_id of this Account.  # noqa: E501
        :type: str
        """

        self._account_id = account_id

    @property
    def type(self):
        """Gets the type of this Account.  # noqa: E501


        :return: The type of this Account.  # noqa: E501
        :rtype: AccountType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this Account.


        :param type: The type of this Account.  # noqa: E501
        :type: AccountType
        """

        self._type = type

    @property
    def code(self):
        """Gets the code of this Account.  # noqa: E501

        Customer defined account code  # noqa: E501

        :return: The code of this Account.  # noqa: E501
        :rtype: str
        """
        return self._code

    @code.setter
    def code(self, code):
        """Sets the code of this Account.

        Customer defined account code  # noqa: E501

        :param code: The code of this Account.  # noqa: E501
        :type: str
        """

        self._code = code

    @property
    def name(self):
        """Gets the name of this Account.  # noqa: E501

        Name of account  # noqa: E501

        :return: The name of this Account.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Account.

        Name of account  # noqa: E501

        :param name: The name of this Account.  # noqa: E501
        :type: str
        """

        self._name = name



# coding: utf-8

"""
    Xero Payroll NZ

    This is the Xero Payroll API for orgs in the NZ region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class LeaveAccrualLine(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"leave_type_id": "str", "number_of_units": "float"}

    attribute_map = {"leave_type_id": "leaveTypeID", "number_of_units": "numberOfUnits"}

    def __init__(self, leave_type_id=None, number_of_units=None):  # noqa: E501
        """LeaveAccrualLine - a model defined in OpenAPI"""  # noqa: E501

        self._leave_type_id = None
        self._number_of_units = None
        self.discriminator = None

        if leave_type_id is not None:
            self.leave_type_id = leave_type_id
        if number_of_units is not None:
            self.number_of_units = number_of_units

    @property
    def leave_type_id(self):
        """Gets the leave_type_id of this LeaveAccrualLine.  # noqa: E501

        Xero identifier for the Leave type  # noqa: E501

        :return: The leave_type_id of this LeaveAccrualLine.  # noqa: E501
        :rtype: str
        """
        return self._leave_type_id

    @leave_type_id.setter
    def leave_type_id(self, leave_type_id):
        """Sets the leave_type_id of this LeaveAccrualLine.

        Xero identifier for the Leave type  # noqa: E501

        :param leave_type_id: The leave_type_id of this LeaveAccrualLine.  # noqa: E501
        :type: str
        """

        self._leave_type_id = leave_type_id

    @property
    def number_of_units(self):
        """Gets the number_of_units of this LeaveAccrualLine.  # noqa: E501

        Leave accrual number of units  # noqa: E501

        :return: The number_of_units of this LeaveAccrualLine.  # noqa: E501
        :rtype: float
        """
        return self._number_of_units

    @number_of_units.setter
    def number_of_units(self, number_of_units):
        """Sets the number_of_units of this LeaveAccrualLine.

        Leave accrual number of units  # noqa: E501

        :param number_of_units: The number_of_units of this LeaveAccrualLine.  # noqa: E501
        :type: float
        """

        self._number_of_units = number_of_units



# -*- coding: utf-8 -*-
"""
    pytest global configuration file.
    Create your global fixtures in this file.
    and pytest shares those with all tests automatically.

    https://docs.pytest.org/en/latest/fixture.html#conftest-py-sharing-fixture-functions
"""
import time

import pytest

from xero_python.api_client import ApiClient
from xero_python.api_client.configuration import Configuration
from xero_python.api_client.oauth2 import OAuth2Token


@pytest.fixture(scope="module")
def vcr_config():
    return {
        # Replace all sensitive data with "XXXXX" in cassettes
        "filter_headers": [("Authorization", "XXXXX"), ("xero-tenant-id", "XXXXX")],
        "filter_post_data_parameters": [
            ("refresh_token", "XXXXX"),
            ("client_id", "XXXXX"),
            ("client_secret", "XXXXX"),
        ],
    }


def get_token_value(key, default=None):
    try:
        from .oauth2_token import token
    except ImportError:
        return default
    else:
        return token.get(key, default)


def get_client_value(key, default=None):
    try:
        from .oauth2_client import client
    except ImportError:
        return default
    else:
        return client.get(key, default)


@pytest.fixture()
def xero_tenant_id():
    return get_client_value("tenant_id") or "xero-tenant-id"


@pytest.fixture()
def xero_client_id():
    return get_client_value("client_id") or "xero-client-id"


@pytest.fixture()
def xero_client_secret():
    return get_client_value("client_secret") or "xero-client-secret"


@pytest.fixture()
def xero_refresh_token():
    return get_token_value("refresh_token") or "xero-refresh-token"


@pytest.fixture()
def xero_id_token():
    return get_token_value("id_token") or "xero-id-token"


@pytest.fixture()
def xero_expires_in():
    return get_token_value("expires_in") or None


@pytest.fixture()
def xero_expires_at():
    return get_token_value("expires_at") or None


@pytest.fixture()
def xero_access_token():
    return (
        get_token_value("access_token") or "put-here-valid-token"
    )  # replace with valid token to re-run all API calls


@pytest.fixture()
def xero_scope():
    return [
        "email",
        "profile",
        "openid",
        "accounting.reports.read",
        "accounting.attachments.read",
        "accounting.settings",
        "accounting.settings.read",
        "accounting.attachments",
        "accounting.transactions",
        "accounting.journals.read",
        "accounting.transactions.read",
        "accounting.contacts",
        "accounting.contacts.read",
        "offline_access",
    ]


@pytest.fixture()
def oauth2_token(
    xero_access_token,
    xero_id_token,
    xero_refresh_token,
    xero_scope,
    xero_expires_in,
    xero_expires_at,
):
    return {
        "access_token": xero_access_token,
        "expires_at": xero_expires_at,
        "expires_in": xero_expires_in,
        "id_token": xero_id_token,
        "refresh_token": xero_refresh_token,
        "scope": xero_scope,
        "token_type": "Bearer",
    }

@pytest.fixture()
def oauth2_token_without_id_token(
    xero_access_token,
    xero_refresh_token,
    xero_scope,
    xero_expires_in,
    xero_expires_at,
):
    return {
        "access_token": xero_access_token,
        "expires_at": xero_expires_at,
        "expires_in": xero_expires_in,
        "refresh_token": xero_refresh_token,
        "scope": xero_scope,
        "token_type": "Bearer",
    }


@pytest.fixture()
def oauth2_refresh_token(oauth2_token, xero_refresh_token):
    """
    similar to valid oauth2_token with expired access token
    """
    oauth2_refresh_token = dict(
        oauth2_token, expires_in=1800, expires_at=time.time() - 1
    )
    return oauth2_refresh_token


@pytest.fixture
def api_config(xero_client_id, xero_client_secret):
    config = Configuration()
    config.oauth2_token = OAuth2Token(
        client_id=xero_client_id, client_secret=xero_client_secret
    )
    return config


@pytest.fixture
def api_client(api_config, oauth2_token):
    def token_getter():
        return oauth2_token

    return ApiClient(
        configuration=api_config, pool_threads=1, oauth2_token_getter=token_getter
    )



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class TimesheetLine(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "earnings_rate_id": "str",
        "tracking_item_id": "str",
        "number_of_units": "list[float]",
        "updated_date_utc": "datetime[ms-format]",
    }

    attribute_map = {
        "earnings_rate_id": "EarningsRateID",
        "tracking_item_id": "TrackingItemID",
        "number_of_units": "NumberOfUnits",
        "updated_date_utc": "UpdatedDateUTC",
    }

    def __init__(
        self,
        earnings_rate_id=None,
        tracking_item_id=None,
        number_of_units=None,
        updated_date_utc=None,
    ):  # noqa: E501
        """TimesheetLine - a model defined in OpenAPI"""  # noqa: E501

        self._earnings_rate_id = None
        self._tracking_item_id = None
        self._number_of_units = None
        self._updated_date_utc = None
        self.discriminator = None

        if earnings_rate_id is not None:
            self.earnings_rate_id = earnings_rate_id
        if tracking_item_id is not None:
            self.tracking_item_id = tracking_item_id
        if number_of_units is not None:
            self.number_of_units = number_of_units
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc

    @property
    def earnings_rate_id(self):
        """Gets the earnings_rate_id of this TimesheetLine.  # noqa: E501

        The Xero identifier for an Earnings Rate  # noqa: E501

        :return: The earnings_rate_id of this TimesheetLine.  # noqa: E501
        :rtype: str
        """
        return self._earnings_rate_id

    @earnings_rate_id.setter
    def earnings_rate_id(self, earnings_rate_id):
        """Sets the earnings_rate_id of this TimesheetLine.

        The Xero identifier for an Earnings Rate  # noqa: E501

        :param earnings_rate_id: The earnings_rate_id of this TimesheetLine.  # noqa: E501
        :type: str
        """

        self._earnings_rate_id = earnings_rate_id

    @property
    def tracking_item_id(self):
        """Gets the tracking_item_id of this TimesheetLine.  # noqa: E501

        The Xero identifier for a Tracking Category. The TrackingOptionID must belong to the TrackingCategory selected as TimesheetCategories under Payroll Settings.  # noqa: E501

        :return: The tracking_item_id of this TimesheetLine.  # noqa: E501
        :rtype: str
        """
        return self._tracking_item_id

    @tracking_item_id.setter
    def tracking_item_id(self, tracking_item_id):
        """Sets the tracking_item_id of this TimesheetLine.

        The Xero identifier for a Tracking Category. The TrackingOptionID must belong to the TrackingCategory selected as TimesheetCategories under Payroll Settings.  # noqa: E501

        :param tracking_item_id: The tracking_item_id of this TimesheetLine.  # noqa: E501
        :type: str
        """

        self._tracking_item_id = tracking_item_id

    @property
    def number_of_units(self):
        """Gets the number_of_units of this TimesheetLine.  # noqa: E501

        The number of units on a timesheet line  # noqa: E501

        :return: The number_of_units of this TimesheetLine.  # noqa: E501
        :rtype: list[float]
        """
        return self._number_of_units

    @number_of_units.setter
    def number_of_units(self, number_of_units):
        """Sets the number_of_units of this TimesheetLine.

        The number of units on a timesheet line  # noqa: E501

        :param number_of_units: The number_of_units of this TimesheetLine.  # noqa: E501
        :type: list[float]
        """

        self._number_of_units = number_of_units

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this TimesheetLine.  # noqa: E501

        Last modified timestamp  # noqa: E501

        :return: The updated_date_utc of this TimesheetLine.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this TimesheetLine.

        Last modified timestamp  # noqa: E501

        :param updated_date_utc: The updated_date_utc of this TimesheetLine.  # noqa: E501
        :type: datetime
        """

        self._updated_date_utc = updated_date_utc



# -*- coding: utf-8 -*-
import json
import time

from xero_python.exceptions import AccessTokenExpiredError


class TokenApi:
    """
    Api class handles interactions with xero token API endpoints
    """

    refresh_token_url = "https://identity.xero.com/connect/token"

    def __init__(self, api_client, client_id, client_secret):
        self.api_client = api_client
        self.client_id = client_id
        self.client_secret = client_secret

    def refresh_token(self, refresh_token, scope):
        """
        Call xero identity API to refresh auth2 access token using refresh token
        :param refresh_token: str auth2 refresh token
        :param scope: list of auth2 scopes
        :return: dictionary with new auth2 token
        """
        post_data = {
            "grant_type": "refresh_token",
            "scope": " ".join(scope),
            "refresh_token": refresh_token,
            "client_id": self.client_id,
            "client_secret": self.client_secret,
        }
        response, status, headers = self.api_client.call_api(
            self.refresh_token_url,
            "POST",
            header_params={
                "Accept": "application/json",
                "Content-Type": "application/x-www-form-urlencoded",
            },
            post_params=post_data,
            auth_settings=None,  # important to prevent infinite recursive loop
            _preload_content=False,
        )
        if status != 200:
            # todo improve error handling
            raise Exception(
                "refresh token status {} {} {!r}".format(status, response, headers)
            )
        # todo validate response is json
        return self.parse_token_response(response)

    def parse_token_response(self, response):
        """
        Parse token data from http response
        :param response: refresh token response
        :return: parsed json data as token dictionary
        """
        # todo decode based on codeset in response content type
        data = response.data.decode("utf-8")
        return json.loads(data)


class OAuth2Token:
    """
    Class to handle oauth2 access and refresh token
    * provides auth header for all `ApiClient` calls
    * implements oauth2 token refresh flow to fetch new valid oauth2 token.
    todo add hooks/signals to:
        * export new refreshed token
        * import new token from outside code
    """

    # refresh access token 60 seconds before it expires
    EXPIRATION_BUFFER_DEFAULT = 60

    def __init__(
        self,
        # rest needed to be able to refresh access_token
        client_id=None,
        client_secret=None,
        expiration_buffer=EXPIRATION_BUFFER_DEFAULT,
    ):
        """
        OAuth2Token constructor,
        client_id and client_server needed to be able to refresh access_token
        :param str client_id: oauth2 client id
        :param str client_secret: oauth2 client secret
        :param int expiration_buffer: refresh access token x seconds prior to expiration
        """
        self.client_id = client_id
        self.client_secret = client_secret
        self.expiration_buffer = expiration_buffer

        # initialize oauth2 token attributes.
        # use self.update_token(...) to set those attributes
        self.token_type = "Bearer"
        self.access_token = self.id_token = self.refresh_token = self.scope = None
        self.expires_at = self.expires_in = None

    def get_auth_settings(self):
        """
        Get auth method ( a callable ) providing dynamic authentication settings
        """
        # return create_auth_settings for api_client to execute
        return self.create_auth_settings

    def create_auth_settings(self, api_client):
        """
        Prepare authorization header for HTTP request
        :param api_client: ApiClient instance used to perform refresh token API call.
        :return: dictionary with authorisation details
        """
        self.update_token(**api_client.get_oauth2_token())
        access_token = self.get_valid_access_token(api_client)
        return {
            "type": "oauth2",
            "in": "header",
            "key": "Authorization",
            "value": "{type} {token}".format(type=self.token_type, token=access_token),
        }

    def is_access_token_valid(self, at_time=None):
        """
        Check current access token valid
        :param at_time: float - timestamp at which token consider valid
        :return: bool token valid ad given time
        """
        at_time = at_time or time.time() + self.expiration_buffer
        if self.expires_at is None:
            return True  # tokens without expiration considered always valid
        return self.expires_at > at_time

    def get_valid_access_token(self, api_client=None, at_time=None):
        """
        Get valid access token (check it's valid and refresh it if needed)
        :param api_client: ApiClient instance used to perform refresh token API call.
        :param at_time: float - timestamp at which token consider valid
        :return: valid auth2 access token string
        :raise: AccessTokenExpiredError
        """
        if not self.is_access_token_valid(at_time):
            if not self.refresh_access_token(api_client):
                raise AccessTokenExpiredError("Access Token has expired")
        return self.access_token

    def can_refresh_access_token(self):
        """
        Check current instance has all data required to perform refresh token API call.
        :return: bool
        """
        return (
            self.refresh_token
            and isinstance(self.scope, (list, tuple))
            and self.client_id
            and self.client_secret
        )

    def refresh_access_token(self, api_client):
        """
        Perform auth2 refresh token call.
        :param api_client:  ApiClient instance used to perform refresh token API call.
        :return: bool - True if success
        :raise: http request related errors
        """
        if not self.can_refresh_access_token():
            return False
        token_api = TokenApi(api_client, self.client_id, self.client_secret)
        new_token = self.fetch_access_token(token_api)
        self.update_token(**new_token)
        api_client.set_oauth2_token(new_token)
        return True

    def update_token(
        self,
        access_token,
        refresh_token,
        scope,
        expires_at,
        expires_in,
        token_type,
        id_token=None,
    ):
        """
        Set new auth2 token details
        :param access_token: str
        :param refresh_token: str
        :param scope: list of strings
        :param expires_at: float timestamp
        :param expires_in: number
        :param token_type: str
        :param id_token: str (optional)
        """
        self.access_token = access_token
        self.expires_at = expires_at
        self.expires_in = expires_in
        self.id_token = id_token
        self.refresh_token = refresh_token
        self.scope = scope
        self.token_type = token_type

    def fetch_access_token(self, token_api, token_valid_from=None):
        """
        Fetch new auth2 token and convert it into auth2 token structure
        :param token_api: TokenApi instance to perform API call.
        :param token_valid_from: float timestamp token expires_in counts from
        :return: dictionary new auth2 token
        """
        token = self.call_refresh_token_api(token_api)
        token_valid_from = token_valid_from or time.time()
        # parse new scope
        new_scope = token.get("scope")
        if new_scope:
            new_scope = str(new_scope).split()
            token["scope"] = new_scope
        # set expires_at
        expires_in = token.get("expires_in")
        if expires_in and "expires_at" not in token:
            token["expires_at"] = token_valid_from + expires_in

        return token

    def call_refresh_token_api(self, token_api):
        """
        Get a new auth2 token using current refresh token
        :param token_api: TokenApi instance
        :return: auth2 token dictionary as received from API.
        """
        return token_api.refresh_token(self.refresh_token, self.scope)



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class SettingsTrackingCategories(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "employee_groups": "SettingsTrackingCategoriesEmployeeGroups",
        "timesheet_categories": "SettingsTrackingCategoriesTimesheetCategories",
    }

    attribute_map = {
        "employee_groups": "EmployeeGroups",
        "timesheet_categories": "TimesheetCategories",
    }

    def __init__(self, employee_groups=None, timesheet_categories=None):  # noqa: E501
        """SettingsTrackingCategories - a model defined in OpenAPI"""  # noqa: E501

        self._employee_groups = None
        self._timesheet_categories = None
        self.discriminator = None

        if employee_groups is not None:
            self.employee_groups = employee_groups
        if timesheet_categories is not None:
            self.timesheet_categories = timesheet_categories

    @property
    def employee_groups(self):
        """Gets the employee_groups of this SettingsTrackingCategories.  # noqa: E501


        :return: The employee_groups of this SettingsTrackingCategories.  # noqa: E501
        :rtype: SettingsTrackingCategoriesEmployeeGroups
        """
        return self._employee_groups

    @employee_groups.setter
    def employee_groups(self, employee_groups):
        """Sets the employee_groups of this SettingsTrackingCategories.


        :param employee_groups: The employee_groups of this SettingsTrackingCategories.  # noqa: E501
        :type: SettingsTrackingCategoriesEmployeeGroups
        """

        self._employee_groups = employee_groups

    @property
    def timesheet_categories(self):
        """Gets the timesheet_categories of this SettingsTrackingCategories.  # noqa: E501


        :return: The timesheet_categories of this SettingsTrackingCategories.  # noqa: E501
        :rtype: SettingsTrackingCategoriesTimesheetCategories
        """
        return self._timesheet_categories

    @timesheet_categories.setter
    def timesheet_categories(self, timesheet_categories):
        """Sets the timesheet_categories of this SettingsTrackingCategories.


        :param timesheet_categories: The timesheet_categories of this SettingsTrackingCategories.  # noqa: E501
        :type: SettingsTrackingCategoriesTimesheetCategories
        """

        self._timesheet_categories = timesheet_categories



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class PayRun(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "payroll_calendar_id": "str",
        "pay_run_id": "str",
        "pay_run_period_start_date": "date[ms-format]",
        "pay_run_period_end_date": "date[ms-format]",
        "pay_run_status": "PayRunStatus",
        "payment_date": "date[ms-format]",
        "payslip_message": "str",
        "updated_date_utc": "datetime[ms-format]",
        "payslips": "list[PayslipSummary]",
        "wages": "float",
        "deductions": "float",
        "tax": "float",
        "super": "float",
        "reimbursement": "float",
        "net_pay": "float",
        "validation_errors": "list[ValidationError]",
    }

    attribute_map = {
        "payroll_calendar_id": "PayrollCalendarID",
        "pay_run_id": "PayRunID",
        "pay_run_period_start_date": "PayRunPeriodStartDate",
        "pay_run_period_end_date": "PayRunPeriodEndDate",
        "pay_run_status": "PayRunStatus",
        "payment_date": "PaymentDate",
        "payslip_message": "PayslipMessage",
        "updated_date_utc": "UpdatedDateUTC",
        "payslips": "Payslips",
        "wages": "Wages",
        "deductions": "Deductions",
        "tax": "Tax",
        "super": "Super",
        "reimbursement": "Reimbursement",
        "net_pay": "NetPay",
        "validation_errors": "ValidationErrors",
    }

    def __init__(
        self,
        payroll_calendar_id=None,
        pay_run_id=None,
        pay_run_period_start_date=None,
        pay_run_period_end_date=None,
        pay_run_status=None,
        payment_date=None,
        payslip_message=None,
        updated_date_utc=None,
        payslips=None,
        wages=None,
        deductions=None,
        tax=None,
        super=None,
        reimbursement=None,
        net_pay=None,
        validation_errors=None,
    ):  # noqa: E501
        """PayRun - a model defined in OpenAPI"""  # noqa: E501

        self._payroll_calendar_id = None
        self._pay_run_id = None
        self._pay_run_period_start_date = None
        self._pay_run_period_end_date = None
        self._pay_run_status = None
        self._payment_date = None
        self._payslip_message = None
        self._updated_date_utc = None
        self._payslips = None
        self._wages = None
        self._deductions = None
        self._tax = None
        self._super = None
        self._reimbursement = None
        self._net_pay = None
        self._validation_errors = None
        self.discriminator = None

        self.payroll_calendar_id = payroll_calendar_id
        if pay_run_id is not None:
            self.pay_run_id = pay_run_id
        if pay_run_period_start_date is not None:
            self.pay_run_period_start_date = pay_run_period_start_date
        if pay_run_period_end_date is not None:
            self.pay_run_period_end_date = pay_run_period_end_date
        if pay_run_status is not None:
            self.pay_run_status = pay_run_status
        if payment_date is not None:
            self.payment_date = payment_date
        if payslip_message is not None:
            self.payslip_message = payslip_message
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc
        if payslips is not None:
            self.payslips = payslips
        if wages is not None:
            self.wages = wages
        if deductions is not None:
            self.deductions = deductions
        if tax is not None:
            self.tax = tax
        if super is not None:
            self.super = super
        if reimbursement is not None:
            self.reimbursement = reimbursement
        if net_pay is not None:
            self.net_pay = net_pay
        if validation_errors is not None:
            self.validation_errors = validation_errors

    @property
    def payroll_calendar_id(self):
        """Gets the payroll_calendar_id of this PayRun.  # noqa: E501

        Xero identifier for pay run  # noqa: E501

        :return: The payroll_calendar_id of this PayRun.  # noqa: E501
        :rtype: str
        """
        return self._payroll_calendar_id

    @payroll_calendar_id.setter
    def payroll_calendar_id(self, payroll_calendar_id):
        """Sets the payroll_calendar_id of this PayRun.

        Xero identifier for pay run  # noqa: E501

        :param payroll_calendar_id: The payroll_calendar_id of this PayRun.  # noqa: E501
        :type: str
        """
        if payroll_calendar_id is None:
            raise ValueError(
                "Invalid value for `payroll_calendar_id`, must not be `None`"
            )  # noqa: E501

        self._payroll_calendar_id = payroll_calendar_id

    @property
    def pay_run_id(self):
        """Gets the pay_run_id of this PayRun.  # noqa: E501

        Xero identifier for pay run  # noqa: E501

        :return: The pay_run_id of this PayRun.  # noqa: E501
        :rtype: str
        """
        return self._pay_run_id

    @pay_run_id.setter
    def pay_run_id(self, pay_run_id):
        """Sets the pay_run_id of this PayRun.

        Xero identifier for pay run  # noqa: E501

        :param pay_run_id: The pay_run_id of this PayRun.  # noqa: E501
        :type: str
        """

        self._pay_run_id = pay_run_id

    @property
    def pay_run_period_start_date(self):
        """Gets the pay_run_period_start_date of this PayRun.  # noqa: E501

        Period Start Date for the PayRun (YYYY-MM-DD)  # noqa: E501

        :return: The pay_run_period_start_date of this PayRun.  # noqa: E501
        :rtype: date
        """
        return self._pay_run_period_start_date

    @pay_run_period_start_date.setter
    def pay_run_period_start_date(self, pay_run_period_start_date):
        """Sets the pay_run_period_start_date of this PayRun.

        Period Start Date for the PayRun (YYYY-MM-DD)  # noqa: E501

        :param pay_run_period_start_date: The pay_run_period_start_date of this PayRun.  # noqa: E501
        :type: date
        """

        self._pay_run_period_start_date = pay_run_period_start_date

    @property
    def pay_run_period_end_date(self):
        """Gets the pay_run_period_end_date of this PayRun.  # noqa: E501

        Period End Date for the PayRun (YYYY-MM-DD)  # noqa: E501

        :return: The pay_run_period_end_date of this PayRun.  # noqa: E501
        :rtype: date
        """
        return self._pay_run_period_end_date

    @pay_run_period_end_date.setter
    def pay_run_period_end_date(self, pay_run_period_end_date):
        """Sets the pay_run_period_end_date of this PayRun.

        Period End Date for the PayRun (YYYY-MM-DD)  # noqa: E501

        :param pay_run_period_end_date: The pay_run_period_end_date of this PayRun.  # noqa: E501
        :type: date
        """

        self._pay_run_period_end_date = pay_run_period_end_date

    @property
    def pay_run_status(self):
        """Gets the pay_run_status of this PayRun.  # noqa: E501


        :return: The pay_run_status of this PayRun.  # noqa: E501
        :rtype: PayRunStatus
        """
        return self._pay_run_status

    @pay_run_status.setter
    def pay_run_status(self, pay_run_status):
        """Sets the pay_run_status of this PayRun.


        :param pay_run_status: The pay_run_status of this PayRun.  # noqa: E501
        :type: PayRunStatus
        """

        self._pay_run_status = pay_run_status

    @property
    def payment_date(self):
        """Gets the payment_date of this PayRun.  # noqa: E501

        Payment Date for the PayRun (YYYY-MM-DD)  # noqa: E501

        :return: The payment_date of this PayRun.  # noqa: E501
        :rtype: date
        """
        return self._payment_date

    @payment_date.setter
    def payment_date(self, payment_date):
        """Sets the payment_date of this PayRun.

        Payment Date for the PayRun (YYYY-MM-DD)  # noqa: E501

        :param payment_date: The payment_date of this PayRun.  # noqa: E501
        :type: date
        """

        self._payment_date = payment_date

    @property
    def payslip_message(self):
        """Gets the payslip_message of this PayRun.  # noqa: E501

        Payslip message for the PayRun  # noqa: E501

        :return: The payslip_message of this PayRun.  # noqa: E501
        :rtype: str
        """
        return self._payslip_message

    @payslip_message.setter
    def payslip_message(self, payslip_message):
        """Sets the payslip_message of this PayRun.

        Payslip message for the PayRun  # noqa: E501

        :param payslip_message: The payslip_message of this PayRun.  # noqa: E501
        :type: str
        """

        self._payslip_message = payslip_message

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this PayRun.  # noqa: E501

        Last modified timestamp  # noqa: E501

        :return: The updated_date_utc of this PayRun.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this PayRun.

        Last modified timestamp  # noqa: E501

        :param updated_date_utc: The updated_date_utc of this PayRun.  # noqa: E501
        :type: datetime
        """

        self._updated_date_utc = updated_date_utc

    @property
    def payslips(self):
        """Gets the payslips of this PayRun.  # noqa: E501

        The payslips in the payrun  # noqa: E501

        :return: The payslips of this PayRun.  # noqa: E501
        :rtype: list[PayslipSummary]
        """
        return self._payslips

    @payslips.setter
    def payslips(self, payslips):
        """Sets the payslips of this PayRun.

        The payslips in the payrun  # noqa: E501

        :param payslips: The payslips of this PayRun.  # noqa: E501
        :type: list[PayslipSummary]
        """

        self._payslips = payslips

    @property
    def wages(self):
        """Gets the wages of this PayRun.  # noqa: E501

        The total Wages for the Payrun  # noqa: E501

        :return: The wages of this PayRun.  # noqa: E501
        :rtype: float
        """
        return self._wages

    @wages.setter
    def wages(self, wages):
        """Sets the wages of this PayRun.

        The total Wages for the Payrun  # noqa: E501

        :param wages: The wages of this PayRun.  # noqa: E501
        :type: float
        """

        self._wages = wages

    @property
    def deductions(self):
        """Gets the deductions of this PayRun.  # noqa: E501

        The total Deductions for the Payrun  # noqa: E501

        :return: The deductions of this PayRun.  # noqa: E501
        :rtype: float
        """
        return self._deductions

    @deductions.setter
    def deductions(self, deductions):
        """Sets the deductions of this PayRun.

        The total Deductions for the Payrun  # noqa: E501

        :param deductions: The deductions of this PayRun.  # noqa: E501
        :type: float
        """

        self._deductions = deductions

    @property
    def tax(self):
        """Gets the tax of this PayRun.  # noqa: E501

        The total Tax for the Payrun  # noqa: E501

        :return: The tax of this PayRun.  # noqa: E501
        :rtype: float
        """
        return self._tax

    @tax.setter
    def tax(self, tax):
        """Sets the tax of this PayRun.

        The total Tax for the Payrun  # noqa: E501

        :param tax: The tax of this PayRun.  # noqa: E501
        :type: float
        """

        self._tax = tax

    @property
    def super(self):
        """Gets the super of this PayRun.  # noqa: E501

        The total Super for the Payrun  # noqa: E501

        :return: The super of this PayRun.  # noqa: E501
        :rtype: float
        """
        return self._super

    @super.setter
    def super(self, super):
        """Sets the super of this PayRun.

        The total Super for the Payrun  # noqa: E501

        :param super: The super of this PayRun.  # noqa: E501
        :type: float
        """

        self._super = super

    @property
    def reimbursement(self):
        """Gets the reimbursement of this PayRun.  # noqa: E501

        The total Reimbursements for the Payrun  # noqa: E501

        :return: The reimbursement of this PayRun.  # noqa: E501
        :rtype: float
        """
        return self._reimbursement

    @reimbursement.setter
    def reimbursement(self, reimbursement):
        """Sets the reimbursement of this PayRun.

        The total Reimbursements for the Payrun  # noqa: E501

        :param reimbursement: The reimbursement of this PayRun.  # noqa: E501
        :type: float
        """

        self._reimbursement = reimbursement

    @property
    def net_pay(self):
        """Gets the net_pay of this PayRun.  # noqa: E501

        The total NetPay for the Payrun  # noqa: E501

        :return: The net_pay of this PayRun.  # noqa: E501
        :rtype: float
        """
        return self._net_pay

    @net_pay.setter
    def net_pay(self, net_pay):
        """Sets the net_pay of this PayRun.

        The total NetPay for the Payrun  # noqa: E501

        :param net_pay: The net_pay of this PayRun.  # noqa: E501
        :type: float
        """

        self._net_pay = net_pay

    @property
    def validation_errors(self):
        """Gets the validation_errors of this PayRun.  # noqa: E501

        Displays array of validation error messages from the API  # noqa: E501

        :return: The validation_errors of this PayRun.  # noqa: E501
        :rtype: list[ValidationError]
        """
        return self._validation_errors

    @validation_errors.setter
    def validation_errors(self, validation_errors):
        """Sets the validation_errors of this PayRun.

        Displays array of validation error messages from the API  # noqa: E501

        :param validation_errors: The validation_errors of this PayRun.  # noqa: E501
        :type: list[ValidationError]
        """

        self._validation_errors = validation_errors



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class ReportRows(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "row_type": "RowType",
        "title": "str",
        "cells": "list[ReportCell]",
        "rows": "list[ReportRow]",
    }

    attribute_map = {
        "row_type": "RowType",
        "title": "Title",
        "cells": "Cells",
        "rows": "Rows",
    }

    def __init__(self, row_type=None, title=None, cells=None, rows=None):  # noqa: E501
        """ReportRows - a model defined in OpenAPI"""  # noqa: E501

        self._row_type = None
        self._title = None
        self._cells = None
        self._rows = None
        self.discriminator = None

        if row_type is not None:
            self.row_type = row_type
        if title is not None:
            self.title = title
        if cells is not None:
            self.cells = cells
        if rows is not None:
            self.rows = rows

    @property
    def row_type(self):
        """Gets the row_type of this ReportRows.  # noqa: E501


        :return: The row_type of this ReportRows.  # noqa: E501
        :rtype: RowType
        """
        return self._row_type

    @row_type.setter
    def row_type(self, row_type):
        """Sets the row_type of this ReportRows.


        :param row_type: The row_type of this ReportRows.  # noqa: E501
        :type: RowType
        """

        self._row_type = row_type

    @property
    def title(self):
        """Gets the title of this ReportRows.  # noqa: E501


        :return: The title of this ReportRows.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this ReportRows.


        :param title: The title of this ReportRows.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def cells(self):
        """Gets the cells of this ReportRows.  # noqa: E501


        :return: The cells of this ReportRows.  # noqa: E501
        :rtype: list[ReportCell]
        """
        return self._cells

    @cells.setter
    def cells(self, cells):
        """Sets the cells of this ReportRows.


        :param cells: The cells of this ReportRows.  # noqa: E501
        :type: list[ReportCell]
        """

        self._cells = cells

    @property
    def rows(self):
        """Gets the rows of this ReportRows.  # noqa: E501


        :return: The rows of this ReportRows.  # noqa: E501
        :rtype: list[ReportRow]
        """
        return self._rows

    @rows.setter
    def rows(self, rows):
        """Sets the rows of this ReportRows.


        :param rows: The rows of this ReportRows.  # noqa: E501
        :type: list[ReportRow]
        """

        self._rows = rows



# coding: utf-8

"""
    Xero Payroll UK

    This is the Xero Payroll API for orgs in the UK region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Employment(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "payroll_calendar_id": "str",
        "start_date": "date",
        "employee_number": "str",
        "ni_category": "str",
    }

    attribute_map = {
        "payroll_calendar_id": "payrollCalendarID",
        "start_date": "startDate",
        "employee_number": "employeeNumber",
        "ni_category": "niCategory",
    }

    def __init__(
        self,
        payroll_calendar_id=None,
        start_date=None,
        employee_number=None,
        ni_category=None,
    ):  # noqa: E501
        """Employment - a model defined in OpenAPI"""  # noqa: E501

        self._payroll_calendar_id = None
        self._start_date = None
        self._employee_number = None
        self._ni_category = None
        self.discriminator = None

        if payroll_calendar_id is not None:
            self.payroll_calendar_id = payroll_calendar_id
        if start_date is not None:
            self.start_date = start_date
        if employee_number is not None:
            self.employee_number = employee_number
        if ni_category is not None:
            self.ni_category = ni_category

    @property
    def payroll_calendar_id(self):
        """Gets the payroll_calendar_id of this Employment.  # noqa: E501

        Xero unique identifier for the payroll calendar of the employee  # noqa: E501

        :return: The payroll_calendar_id of this Employment.  # noqa: E501
        :rtype: str
        """
        return self._payroll_calendar_id

    @payroll_calendar_id.setter
    def payroll_calendar_id(self, payroll_calendar_id):
        """Sets the payroll_calendar_id of this Employment.

        Xero unique identifier for the payroll calendar of the employee  # noqa: E501

        :param payroll_calendar_id: The payroll_calendar_id of this Employment.  # noqa: E501
        :type: str
        """

        self._payroll_calendar_id = payroll_calendar_id

    @property
    def start_date(self):
        """Gets the start_date of this Employment.  # noqa: E501

        Start date of the employment (YYYY-MM-DD)  # noqa: E501

        :return: The start_date of this Employment.  # noqa: E501
        :rtype: date
        """
        return self._start_date

    @start_date.setter
    def start_date(self, start_date):
        """Sets the start_date of this Employment.

        Start date of the employment (YYYY-MM-DD)  # noqa: E501

        :param start_date: The start_date of this Employment.  # noqa: E501
        :type: date
        """

        self._start_date = start_date

    @property
    def employee_number(self):
        """Gets the employee_number of this Employment.  # noqa: E501

        The employment number of the employee  # noqa: E501

        :return: The employee_number of this Employment.  # noqa: E501
        :rtype: str
        """
        return self._employee_number

    @employee_number.setter
    def employee_number(self, employee_number):
        """Sets the employee_number of this Employment.

        The employment number of the employee  # noqa: E501

        :param employee_number: The employee_number of this Employment.  # noqa: E501
        :type: str
        """

        self._employee_number = employee_number

    @property
    def ni_category(self):
        """Gets the ni_category of this Employment.  # noqa: E501

        The NI Category of the employee  # noqa: E501

        :return: The ni_category of this Employment.  # noqa: E501
        :rtype: str
        """
        return self._ni_category

    @ni_category.setter
    def ni_category(self, ni_category):
        """Sets the ni_category of this Employment.

        The NI Category of the employee  # noqa: E501

        :param ni_category: The ni_category of this Employment.  # noqa: E501
        :type: str
        """
        allowed_values = ["A", "B", "C", "H", "J", "M", "Z", "X", "None"]  # noqa: E501

        if ni_category:
            if ni_category not in allowed_values:
                raise ValueError(
                    "Invalid value for `ni_category` ({0}), must be one of {1}".format(  # noqa: E501
                        ni_category, allowed_values
                    )
                )

        self._ni_category = ni_category



# coding: utf-8

"""
    Xero Files API

    These endpoints are specific to Xero Files API  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class UploadObject(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "body": "str",
        "name": "str",
        "filename": "str",
        "mime_type": "str",
    }

    attribute_map = {
        "body": "body",
        "name": "name",
        "filename": "filename",
        "mime_type": "mimeType",
    }

    def __init__(
        self, body=None, name=None, filename=None, mime_type=None
    ):  # noqa: E501
        """UploadObject - a model defined in OpenAPI"""  # noqa: E501

        self._body = None
        self._name = None
        self._filename = None
        self._mime_type = None
        self.discriminator = None

        self.body = body
        self.name = name
        self.filename = filename
        if mime_type is not None:
            self.mime_type = mime_type

    @property
    def body(self):
        """Gets the body of this UploadObject.  # noqa: E501


        :return: The body of this UploadObject.  # noqa: E501
        :rtype: str
        """
        return self._body

    @body.setter
    def body(self, body):
        """Sets the body of this UploadObject.


        :param body: The body of this UploadObject.  # noqa: E501
        :type: str
        """
        if body is None:
            raise ValueError(
                "Invalid value for `body`, must not be `None`"
            )  # noqa: E501

        self._body = body

    @property
    def name(self):
        """Gets the name of this UploadObject.  # noqa: E501

        exact name of the file you are uploading  # noqa: E501

        :return: The name of this UploadObject.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this UploadObject.

        exact name of the file you are uploading  # noqa: E501

        :param name: The name of this UploadObject.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError(
                "Invalid value for `name`, must not be `None`"
            )  # noqa: E501

        self._name = name

    @property
    def filename(self):
        """Gets the filename of this UploadObject.  # noqa: E501


        :return: The filename of this UploadObject.  # noqa: E501
        :rtype: str
        """
        return self._filename

    @filename.setter
    def filename(self, filename):
        """Sets the filename of this UploadObject.


        :param filename: The filename of this UploadObject.  # noqa: E501
        :type: str
        """
        if filename is None:
            raise ValueError(
                "Invalid value for `filename`, must not be `None`"
            )  # noqa: E501

        self._filename = filename

    @property
    def mime_type(self):
        """Gets the mime_type of this UploadObject.  # noqa: E501


        :return: The mime_type of this UploadObject.  # noqa: E501
        :rtype: str
        """
        return self._mime_type

    @mime_type.setter
    def mime_type(self, mime_type):
        """Sets the mime_type of this UploadObject.


        :param mime_type: The mime_type of this UploadObject.  # noqa: E501
        :type: str
        """

        self._mime_type = mime_type



# -*- coding: utf-8 -*-
import re
from datetime import datetime, time, date
from enum import Enum
from functools import singledispatch

from dateutil import tz

from xero_python.models import BaseModel
from xero_python.single_dispatch_str import single_dispatch_str

DICT_DATA_TYPE = re.compile(r"^dict(?:\[(.*)\])?$")
LIST_DATA_TYPE = re.compile(r"^list(?:\[(.*)\])?$")
TUPLE_DATA_TYPE = re.compile(r"^tuple(?:\[(.*)\])?$")


def data_type(value, explicit_type=None):
    """Get serialization data type for given value

    :param value: instance to serialize
    :param string explicit_type: explicit serialization type for value
    :return: string value serialization type
    """
    return str(explicit_type or type(value).__name__)


def serialize_routing(value, explicit_type=None):
    """Custom logic to find matching serialize implementation and
       returns it's unique registration string key
    :param value: instance to serialize
    :param explicit_type: explicit serialization type for value

    :return: str key to find proper serialize implementation
    """
    value_type = data_type(value, explicit_type)
    if DICT_DATA_TYPE.match(value_type):
        return "dict"
    if LIST_DATA_TYPE.match(value_type):
        return "list"
    if TUPLE_DATA_TYPE.match(value_type):
        return "tuple"
    return value_type


@single_dispatch_str(key=serialize_routing)
def serialize(value, explicit_type=None):
    """Serializes data into an json serializable object.
       Default implementation search model class for given data type

    :param value: instance to serialize
    :param string explicit_type: explicit serialization type for value
    :return: serialized object
    """
    if explicit_type and explicit_type != data_type(value):
        raise ValueError(
            "Can't serialize value type {!r} to explicit type {!r}".format(
                data_type(value), explicit_type
            )
        )

    return serialize_model(value)


@serialize.register("dict")
def serialize_dict(value, explicit_type=None):
    """Serializes dict data into an json serializable object.

    :param dict value: instance to serialize
    :param string explicit_type: explicit serialization type for value
    :return: serialized object
    """
    value_type = data_type(value, explicit_type)
    try:
        sub_type = DICT_DATA_TYPE.match(value_type).group(1)
    except AttributeError:
        raise ValueError("Can't serialize value type {!r}".format(value_type))
    else:
        return {key: serialize(sub_value, sub_type) for key, sub_value in value.items()}


@serialize.register("list")
def serialize_list(value, explicit_type=None):
    """Serializes list data into an json serializable object.

    :param list value: instance to serialize
    :param string explicit_type: explicit serialization type for value
    :return: serialized object
    """
    value_type = data_type(value, explicit_type)
    try:
        sub_type = LIST_DATA_TYPE.match(value_type).group(1)
    except AttributeError:
        raise ValueError("Can't serialize value type {!r}".format(value_type))
    else:
        return [serialize(sub_value, sub_type) for sub_value in value]


@serialize.register("tuple")
def serialize_tuple(value, explicit_type=None):
    """Serializes tuple data into an json serializable object.

    :param tuple value: instance to serialize
    :param string explicit_type: explicit serialization type for value
    :return: serialized object
    """
    value_type = data_type(value, explicit_type)
    try:
        sub_type = TUPLE_DATA_TYPE.match(value_type).group(1)
    except AttributeError:
        raise ValueError("Can't serialize value type {!r}".format(value_type))
    else:
        return tuple(serialize(sub_value, sub_type) for sub_value in value)


@serialize.register("str")
@serialize.register("bool")
@serialize.register("int")
@serialize.register("bytes")
def serialize_pass_through(value, explicit_type=None):
    """Serializes data by passing them directly to json serializable object.

    :param value: instance to serialize
    :param string explicit_type: explicit serialization type for value
    :return: serialized object
    """
    return value


@serialize.register("float")
@serialize.register("Decimal")
def serialize_float(value, explicit_type=None):
    """Serializes float/decimal value as float number for json serializable object.

    :param value: instance to serialize
    :param string explicit_type: explicit serialization type for value
    :return: serialized object
    """
    return float(value)


@serialize.register("date")
@serialize.register("datetime")
def serialize_datetime(value, explicit_type=None):
    """Serializes date/datetime value as iso string for json serializable object.

    :param date value: instance to serialize
    :param string explicit_type: explicit serialization type for value
    :return: serialized object
    """
    return value.isoformat()


@serialize.register("datetime[ms-format]")
def serialize_datetime_ms(value, explicit_type=None):
    """Serializes datetime value as MS json date string

    :param datetime value: instance to serialize
    :param string explicit_type: explicit serialization type for value
    :return: serialized object
    """
    tz_str = value.strftime("%z")
    timestamp_s = value.timestamp()
    timestamp_ms = int(timestamp_s * 1000)
    return "/Date({}{})/".format(timestamp_ms, tz_str)


@serialize.register("date[ms-format]")
def serialize_date_ms(value, explicit_type=None):
    """Serializes date value as MS json date string

    :param date value: instance to serialize
    :param string explicit_type: explicit serialization type for value
    :return: serialized object
    """
    if isinstance(value, datetime):
        datetime_value = value
    elif isinstance(value, date):
        # combine date with 00:00 UTC time to get correct unix timestamp
        datetime_value = datetime.combine(value, time(tzinfo=tz.UTC))
    else:
        raise ValueError("Can't serialize {!r} into Microsoft date json format")

    timestamp_s = datetime_value.timestamp()
    timestamp_ms = int(timestamp_s * 1000)
    return "/Date({})/".format(timestamp_ms)


@singledispatch
def serialize_model(model):
    """Serializes api model into an json serializable object.

    :param model: BaseModel instance to serialize
    :return: serialized object
    """
    raise ValueError("Can't serialize value type {!r}".format(data_type(model)))


@serialize_model.register(BaseModel)
def serialize_base_model(model):
    """Serializes api model into an json serializable object.

    :param model: BaseModel instance to serialize
    :return: serialized object
    """
    serialized = {}
    for attr_name, attr_type in model.openapi_types.items():
        attr_value = getattr(model, attr_name)
        if attr_value is not None:  # TODO how do you null given field
            key = model.attribute_map[attr_name]
            attr_value = serialize(attr_value, attr_type)
            serialized[key] = attr_value

    return serialized


@serialize_model.register(Enum)
def serialize_enum_model(model):
    """Serializes api model into an json serializable object.

    :param model: BaseModel instance to serialize
    :return: serialized object
    """
    return model.value



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class ReportAttribute(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"id": "str", "value": "str"}

    attribute_map = {"id": "Id", "value": "Value"}

    def __init__(self, id=None, value=None):  # noqa: E501
        """ReportAttribute - a model defined in OpenAPI"""  # noqa: E501

        self._id = None
        self._value = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if value is not None:
            self.value = value

    @property
    def id(self):
        """Gets the id of this ReportAttribute.  # noqa: E501


        :return: The id of this ReportAttribute.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ReportAttribute.


        :param id: The id of this ReportAttribute.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def value(self):
        """Gets the value of this ReportAttribute.  # noqa: E501


        :return: The value of this ReportAttribute.  # noqa: E501
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this ReportAttribute.


        :param value: The value of this ReportAttribute.  # noqa: E501
        :type: str
        """

        self._value = value



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class ConversionDate(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"month": "int", "year": "int"}

    attribute_map = {"month": "Month", "year": "Year"}

    def __init__(self, month=None, year=None):  # noqa: E501
        """ConversionDate - a model defined in OpenAPI"""  # noqa: E501

        self._month = None
        self._year = None
        self.discriminator = None

        if month is not None:
            self.month = month
        if year is not None:
            self.year = year

    @property
    def month(self):
        """Gets the month of this ConversionDate.  # noqa: E501

        The month the organisation starts using Xero. Value is an integer between 1 and 12  # noqa: E501

        :return: The month of this ConversionDate.  # noqa: E501
        :rtype: int
        """
        return self._month

    @month.setter
    def month(self, month):
        """Sets the month of this ConversionDate.

        The month the organisation starts using Xero. Value is an integer between 1 and 12  # noqa: E501

        :param month: The month of this ConversionDate.  # noqa: E501
        :type: int
        """

        self._month = month

    @property
    def year(self):
        """Gets the year of this ConversionDate.  # noqa: E501

        The year the organisation starts using Xero. Value is an integer greater than 2006  # noqa: E501

        :return: The year of this ConversionDate.  # noqa: E501
        :rtype: int
        """
        return self._year

    @year.setter
    def year(self, year):
        """Sets the year of this ConversionDate.

        The year the organisation starts using Xero. Value is an integer greater than 2006  # noqa: E501

        :param year: The year of this ConversionDate.  # noqa: E501
        :type: int
        """

        self._year = year



# coding: utf-8

"""
    Xero Assets API

    The Assets API exposes fixed asset related functions of the Xero Accounting application and can be used for a variety of purposes such as creating assets, retrieving asset valuations etc.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Asset(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "asset_id": "str",
        "asset_name": "str",
        "asset_type_id": "str",
        "asset_number": "str",
        "purchase_date": "date",
        "purchase_price": "float",
        "disposal_date": "date",
        "disposal_price": "float",
        "asset_status": "AssetStatus",
        "warranty_expiry_date": "str",
        "serial_number": "str",
        "book_depreciation_setting": "BookDepreciationSetting",
        "book_depreciation_detail": "BookDepreciationDetail",
        "can_rollback": "bool",
        "accounting_book_value": "float",
        "is_delete_enabled_for_date": "bool",
    }

    attribute_map = {
        "asset_id": "assetId",
        "asset_name": "assetName",
        "asset_type_id": "assetTypeId",
        "asset_number": "assetNumber",
        "purchase_date": "purchaseDate",
        "purchase_price": "purchasePrice",
        "disposal_date": "disposalDate",
        "disposal_price": "disposalPrice",
        "asset_status": "assetStatus",
        "warranty_expiry_date": "warrantyExpiryDate",
        "serial_number": "serialNumber",
        "book_depreciation_setting": "bookDepreciationSetting",
        "book_depreciation_detail": "bookDepreciationDetail",
        "can_rollback": "canRollback",
        "accounting_book_value": "accountingBookValue",
        "is_delete_enabled_for_date": "isDeleteEnabledForDate",
    }

    def __init__(
        self,
        asset_id=None,
        asset_name=None,
        asset_type_id=None,
        asset_number=None,
        purchase_date=None,
        purchase_price=None,
        disposal_date=None,
        disposal_price=None,
        asset_status=None,
        warranty_expiry_date=None,
        serial_number=None,
        book_depreciation_setting=None,
        book_depreciation_detail=None,
        can_rollback=None,
        accounting_book_value=None,
        is_delete_enabled_for_date=None,
    ):  # noqa: E501
        """Asset - a model defined in OpenAPI"""  # noqa: E501

        self._asset_id = None
        self._asset_name = None
        self._asset_type_id = None
        self._asset_number = None
        self._purchase_date = None
        self._purchase_price = None
        self._disposal_date = None
        self._disposal_price = None
        self._asset_status = None
        self._warranty_expiry_date = None
        self._serial_number = None
        self._book_depreciation_setting = None
        self._book_depreciation_detail = None
        self._can_rollback = None
        self._accounting_book_value = None
        self._is_delete_enabled_for_date = None
        self.discriminator = None

        if asset_id is not None:
            self.asset_id = asset_id
        self.asset_name = asset_name
        if asset_type_id is not None:
            self.asset_type_id = asset_type_id
        if asset_number is not None:
            self.asset_number = asset_number
        if purchase_date is not None:
            self.purchase_date = purchase_date
        if purchase_price is not None:
            self.purchase_price = purchase_price
        if disposal_date is not None:
            self.disposal_date = disposal_date
        if disposal_price is not None:
            self.disposal_price = disposal_price
        if asset_status is not None:
            self.asset_status = asset_status
        if warranty_expiry_date is not None:
            self.warranty_expiry_date = warranty_expiry_date
        if serial_number is not None:
            self.serial_number = serial_number
        if book_depreciation_setting is not None:
            self.book_depreciation_setting = book_depreciation_setting
        if book_depreciation_detail is not None:
            self.book_depreciation_detail = book_depreciation_detail
        if can_rollback is not None:
            self.can_rollback = can_rollback
        if accounting_book_value is not None:
            self.accounting_book_value = accounting_book_value
        if is_delete_enabled_for_date is not None:
            self.is_delete_enabled_for_date = is_delete_enabled_for_date

    @property
    def asset_id(self):
        """Gets the asset_id of this Asset.  # noqa: E501

        The Xero-generated Id for the asset  # noqa: E501

        :return: The asset_id of this Asset.  # noqa: E501
        :rtype: str
        """
        return self._asset_id

    @asset_id.setter
    def asset_id(self, asset_id):
        """Sets the asset_id of this Asset.

        The Xero-generated Id for the asset  # noqa: E501

        :param asset_id: The asset_id of this Asset.  # noqa: E501
        :type: str
        """

        self._asset_id = asset_id

    @property
    def asset_name(self):
        """Gets the asset_name of this Asset.  # noqa: E501

        The name of the asset  # noqa: E501

        :return: The asset_name of this Asset.  # noqa: E501
        :rtype: str
        """
        return self._asset_name

    @asset_name.setter
    def asset_name(self, asset_name):
        """Sets the asset_name of this Asset.

        The name of the asset  # noqa: E501

        :param asset_name: The asset_name of this Asset.  # noqa: E501
        :type: str
        """
        if asset_name is None:
            raise ValueError(
                "Invalid value for `asset_name`, must not be `None`"
            )  # noqa: E501

        self._asset_name = asset_name

    @property
    def asset_type_id(self):
        """Gets the asset_type_id of this Asset.  # noqa: E501

        The Xero-generated Id for the asset type  # noqa: E501

        :return: The asset_type_id of this Asset.  # noqa: E501
        :rtype: str
        """
        return self._asset_type_id

    @asset_type_id.setter
    def asset_type_id(self, asset_type_id):
        """Sets the asset_type_id of this Asset.

        The Xero-generated Id for the asset type  # noqa: E501

        :param asset_type_id: The asset_type_id of this Asset.  # noqa: E501
        :type: str
        """

        self._asset_type_id = asset_type_id

    @property
    def asset_number(self):
        """Gets the asset_number of this Asset.  # noqa: E501

        Must be unique.  # noqa: E501

        :return: The asset_number of this Asset.  # noqa: E501
        :rtype: str
        """
        return self._asset_number

    @asset_number.setter
    def asset_number(self, asset_number):
        """Sets the asset_number of this Asset.

        Must be unique.  # noqa: E501

        :param asset_number: The asset_number of this Asset.  # noqa: E501
        :type: str
        """

        self._asset_number = asset_number

    @property
    def purchase_date(self):
        """Gets the purchase_date of this Asset.  # noqa: E501

        The date the asset was purchased YYYY-MM-DD  # noqa: E501

        :return: The purchase_date of this Asset.  # noqa: E501
        :rtype: date
        """
        return self._purchase_date

    @purchase_date.setter
    def purchase_date(self, purchase_date):
        """Sets the purchase_date of this Asset.

        The date the asset was purchased YYYY-MM-DD  # noqa: E501

        :param purchase_date: The purchase_date of this Asset.  # noqa: E501
        :type: date
        """

        self._purchase_date = purchase_date

    @property
    def purchase_price(self):
        """Gets the purchase_price of this Asset.  # noqa: E501

        The purchase price of the asset  # noqa: E501

        :return: The purchase_price of this Asset.  # noqa: E501
        :rtype: float
        """
        return self._purchase_price

    @purchase_price.setter
    def purchase_price(self, purchase_price):
        """Sets the purchase_price of this Asset.

        The purchase price of the asset  # noqa: E501

        :param purchase_price: The purchase_price of this Asset.  # noqa: E501
        :type: float
        """

        self._purchase_price = purchase_price

    @property
    def disposal_date(self):
        """Gets the disposal_date of this Asset.  # noqa: E501

        The date the asset was disposed  # noqa: E501

        :return: The disposal_date of this Asset.  # noqa: E501
        :rtype: date
        """
        return self._disposal_date

    @disposal_date.setter
    def disposal_date(self, disposal_date):
        """Sets the disposal_date of this Asset.

        The date the asset was disposed  # noqa: E501

        :param disposal_date: The disposal_date of this Asset.  # noqa: E501
        :type: date
        """

        self._disposal_date = disposal_date

    @property
    def disposal_price(self):
        """Gets the disposal_price of this Asset.  # noqa: E501

        The price the asset was disposed at  # noqa: E501

        :return: The disposal_price of this Asset.  # noqa: E501
        :rtype: float
        """
        return self._disposal_price

    @disposal_price.setter
    def disposal_price(self, disposal_price):
        """Sets the disposal_price of this Asset.

        The price the asset was disposed at  # noqa: E501

        :param disposal_price: The disposal_price of this Asset.  # noqa: E501
        :type: float
        """

        self._disposal_price = disposal_price

    @property
    def asset_status(self):
        """Gets the asset_status of this Asset.  # noqa: E501


        :return: The asset_status of this Asset.  # noqa: E501
        :rtype: AssetStatus
        """
        return self._asset_status

    @asset_status.setter
    def asset_status(self, asset_status):
        """Sets the asset_status of this Asset.


        :param asset_status: The asset_status of this Asset.  # noqa: E501
        :type: AssetStatus
        """

        self._asset_status = asset_status

    @property
    def warranty_expiry_date(self):
        """Gets the warranty_expiry_date of this Asset.  # noqa: E501

        The date the asset’s warranty expires (if needed) YYYY-MM-DD  # noqa: E501

        :return: The warranty_expiry_date of this Asset.  # noqa: E501
        :rtype: str
        """
        return self._warranty_expiry_date

    @warranty_expiry_date.setter
    def warranty_expiry_date(self, warranty_expiry_date):
        """Sets the warranty_expiry_date of this Asset.

        The date the asset’s warranty expires (if needed) YYYY-MM-DD  # noqa: E501

        :param warranty_expiry_date: The warranty_expiry_date of this Asset.  # noqa: E501
        :type: str
        """

        self._warranty_expiry_date = warranty_expiry_date

    @property
    def serial_number(self):
        """Gets the serial_number of this Asset.  # noqa: E501

        The asset's serial number  # noqa: E501

        :return: The serial_number of this Asset.  # noqa: E501
        :rtype: str
        """
        return self._serial_number

    @serial_number.setter
    def serial_number(self, serial_number):
        """Sets the serial_number of this Asset.

        The asset's serial number  # noqa: E501

        :param serial_number: The serial_number of this Asset.  # noqa: E501
        :type: str
        """

        self._serial_number = serial_number

    @property
    def book_depreciation_setting(self):
        """Gets the book_depreciation_setting of this Asset.  # noqa: E501


        :return: The book_depreciation_setting of this Asset.  # noqa: E501
        :rtype: BookDepreciationSetting
        """
        return self._book_depreciation_setting

    @book_depreciation_setting.setter
    def book_depreciation_setting(self, book_depreciation_setting):
        """Sets the book_depreciation_setting of this Asset.


        :param book_depreciation_setting: The book_depreciation_setting of this Asset.  # noqa: E501
        :type: BookDepreciationSetting
        """

        self._book_depreciation_setting = book_depreciation_setting

    @property
    def book_depreciation_detail(self):
        """Gets the book_depreciation_detail of this Asset.  # noqa: E501


        :return: The book_depreciation_detail of this Asset.  # noqa: E501
        :rtype: BookDepreciationDetail
        """
        return self._book_depreciation_detail

    @book_depreciation_detail.setter
    def book_depreciation_detail(self, book_depreciation_detail):
        """Sets the book_depreciation_detail of this Asset.


        :param book_depreciation_detail: The book_depreciation_detail of this Asset.  # noqa: E501
        :type: BookDepreciationDetail
        """

        self._book_depreciation_detail = book_depreciation_detail

    @property
    def can_rollback(self):
        """Gets the can_rollback of this Asset.  # noqa: E501

        Boolean to indicate whether depreciation can be rolled back for this asset individually. This is true if it doesn't have 'legacy' journal entries and if there is no lock period that would prevent this asset from rolling back.  # noqa: E501

        :return: The can_rollback of this Asset.  # noqa: E501
        :rtype: bool
        """
        return self._can_rollback

    @can_rollback.setter
    def can_rollback(self, can_rollback):
        """Sets the can_rollback of this Asset.

        Boolean to indicate whether depreciation can be rolled back for this asset individually. This is true if it doesn't have 'legacy' journal entries and if there is no lock period that would prevent this asset from rolling back.  # noqa: E501

        :param can_rollback: The can_rollback of this Asset.  # noqa: E501
        :type: bool
        """

        self._can_rollback = can_rollback

    @property
    def accounting_book_value(self):
        """Gets the accounting_book_value of this Asset.  # noqa: E501

        The accounting value of the asset  # noqa: E501

        :return: The accounting_book_value of this Asset.  # noqa: E501
        :rtype: float
        """
        return self._accounting_book_value

    @accounting_book_value.setter
    def accounting_book_value(self, accounting_book_value):
        """Sets the accounting_book_value of this Asset.

        The accounting value of the asset  # noqa: E501

        :param accounting_book_value: The accounting_book_value of this Asset.  # noqa: E501
        :type: float
        """

        self._accounting_book_value = accounting_book_value

    @property
    def is_delete_enabled_for_date(self):
        """Gets the is_delete_enabled_for_date of this Asset.  # noqa: E501

        Boolean to indicate whether delete is enabled  # noqa: E501

        :return: The is_delete_enabled_for_date of this Asset.  # noqa: E501
        :rtype: bool
        """
        return self._is_delete_enabled_for_date

    @is_delete_enabled_for_date.setter
    def is_delete_enabled_for_date(self, is_delete_enabled_for_date):
        """Sets the is_delete_enabled_for_date of this Asset.

        Boolean to indicate whether delete is enabled  # noqa: E501

        :param is_delete_enabled_for_date: The is_delete_enabled_for_date of this Asset.  # noqa: E501
        :type: bool
        """

        self._is_delete_enabled_for_date = is_delete_enabled_for_date



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class SuperFunds(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"super_funds": "list[SuperFund]"}

    attribute_map = {"super_funds": "SuperFunds"}

    def __init__(self, super_funds=None):  # noqa: E501
        """SuperFunds - a model defined in OpenAPI"""  # noqa: E501

        self._super_funds = None
        self.discriminator = None

        if super_funds is not None:
            self.super_funds = super_funds

    @property
    def super_funds(self):
        """Gets the super_funds of this SuperFunds.  # noqa: E501


        :return: The super_funds of this SuperFunds.  # noqa: E501
        :rtype: list[SuperFund]
        """
        return self._super_funds

    @super_funds.setter
    def super_funds(self, super_funds):
        """Sets the super_funds of this SuperFunds.


        :param super_funds: The super_funds of this SuperFunds.  # noqa: E501
        :type: list[SuperFund]
        """

        self._super_funds = super_funds



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class TaxLine(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "payslip_tax_line_id": "str",
        "amount": "float",
        "tax_type_name": "str",
        "description": "str",
        "manual_tax_type": "ManualTaxType",
        "liability_account": "str",
    }

    attribute_map = {
        "payslip_tax_line_id": "PayslipTaxLineID",
        "amount": "Amount",
        "tax_type_name": "TaxTypeName",
        "description": "Description",
        "manual_tax_type": "ManualTaxType",
        "liability_account": "LiabilityAccount",
    }

    def __init__(
        self,
        payslip_tax_line_id=None,
        amount=None,
        tax_type_name=None,
        description=None,
        manual_tax_type=None,
        liability_account=None,
    ):  # noqa: E501
        """TaxLine - a model defined in OpenAPI"""  # noqa: E501

        self._payslip_tax_line_id = None
        self._amount = None
        self._tax_type_name = None
        self._description = None
        self._manual_tax_type = None
        self._liability_account = None
        self.discriminator = None

        if payslip_tax_line_id is not None:
            self.payslip_tax_line_id = payslip_tax_line_id
        if amount is not None:
            self.amount = amount
        if tax_type_name is not None:
            self.tax_type_name = tax_type_name
        if description is not None:
            self.description = description
        if manual_tax_type is not None:
            self.manual_tax_type = manual_tax_type
        if liability_account is not None:
            self.liability_account = liability_account

    @property
    def payslip_tax_line_id(self):
        """Gets the payslip_tax_line_id of this TaxLine.  # noqa: E501

        Xero identifier for payslip tax line ID.  # noqa: E501

        :return: The payslip_tax_line_id of this TaxLine.  # noqa: E501
        :rtype: str
        """
        return self._payslip_tax_line_id

    @payslip_tax_line_id.setter
    def payslip_tax_line_id(self, payslip_tax_line_id):
        """Sets the payslip_tax_line_id of this TaxLine.

        Xero identifier for payslip tax line ID.  # noqa: E501

        :param payslip_tax_line_id: The payslip_tax_line_id of this TaxLine.  # noqa: E501
        :type: str
        """

        self._payslip_tax_line_id = payslip_tax_line_id

    @property
    def amount(self):
        """Gets the amount of this TaxLine.  # noqa: E501

        The tax line amount  # noqa: E501

        :return: The amount of this TaxLine.  # noqa: E501
        :rtype: float
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """Sets the amount of this TaxLine.

        The tax line amount  # noqa: E501

        :param amount: The amount of this TaxLine.  # noqa: E501
        :type: float
        """

        self._amount = amount

    @property
    def tax_type_name(self):
        """Gets the tax_type_name of this TaxLine.  # noqa: E501

        Name of the tax type.  # noqa: E501

        :return: The tax_type_name of this TaxLine.  # noqa: E501
        :rtype: str
        """
        return self._tax_type_name

    @tax_type_name.setter
    def tax_type_name(self, tax_type_name):
        """Sets the tax_type_name of this TaxLine.

        Name of the tax type.  # noqa: E501

        :param tax_type_name: The tax_type_name of this TaxLine.  # noqa: E501
        :type: str
        """

        self._tax_type_name = tax_type_name

    @property
    def description(self):
        """Gets the description of this TaxLine.  # noqa: E501

        Description of the tax line.  # noqa: E501

        :return: The description of this TaxLine.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this TaxLine.

        Description of the tax line.  # noqa: E501

        :param description: The description of this TaxLine.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def manual_tax_type(self):
        """Gets the manual_tax_type of this TaxLine.  # noqa: E501


        :return: The manual_tax_type of this TaxLine.  # noqa: E501
        :rtype: ManualTaxType
        """
        return self._manual_tax_type

    @manual_tax_type.setter
    def manual_tax_type(self, manual_tax_type):
        """Sets the manual_tax_type of this TaxLine.


        :param manual_tax_type: The manual_tax_type of this TaxLine.  # noqa: E501
        :type: ManualTaxType
        """

        self._manual_tax_type = manual_tax_type

    @property
    def liability_account(self):
        """Gets the liability_account of this TaxLine.  # noqa: E501

        The tax line liability account code. For posted pay run you should be able to see liability account code  # noqa: E501

        :return: The liability_account of this TaxLine.  # noqa: E501
        :rtype: str
        """
        return self._liability_account

    @liability_account.setter
    def liability_account(self, liability_account):
        """Sets the liability_account of this TaxLine.

        The tax line liability account code. For posted pay run you should be able to see liability account code  # noqa: E501

        :param liability_account: The liability_account of this TaxLine.  # noqa: E501
        :type: str
        """

        self._liability_account = liability_account



# coding: utf-8

"""
    Xero Payroll NZ

    This is the Xero Payroll API for orgs in the NZ region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class PaymentMethod(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"payment_method": "str", "bank_accounts": "list[BankAccount]"}

    attribute_map = {"payment_method": "paymentMethod", "bank_accounts": "bankAccounts"}

    def __init__(self, payment_method=None, bank_accounts=None):  # noqa: E501
        """PaymentMethod - a model defined in OpenAPI"""  # noqa: E501

        self._payment_method = None
        self._bank_accounts = None
        self.discriminator = None

        if payment_method is not None:
            self.payment_method = payment_method
        if bank_accounts is not None:
            self.bank_accounts = bank_accounts

    @property
    def payment_method(self):
        """Gets the payment_method of this PaymentMethod.  # noqa: E501

        The payment method code  # noqa: E501

        :return: The payment_method of this PaymentMethod.  # noqa: E501
        :rtype: str
        """
        return self._payment_method

    @payment_method.setter
    def payment_method(self, payment_method):
        """Sets the payment_method of this PaymentMethod.

        The payment method code  # noqa: E501

        :param payment_method: The payment_method of this PaymentMethod.  # noqa: E501
        :type: str
        """
        allowed_values = ["Cheque", "Electronically", "Manual", "None"]  # noqa: E501

        if payment_method:
            if payment_method not in allowed_values:
                raise ValueError(
                    "Invalid value for `payment_method` ({0}), must be one of {1}".format(  # noqa: E501
                        payment_method, allowed_values
                    )
                )

        self._payment_method = payment_method

    @property
    def bank_accounts(self):
        """Gets the bank_accounts of this PaymentMethod.  # noqa: E501


        :return: The bank_accounts of this PaymentMethod.  # noqa: E501
        :rtype: list[BankAccount]
        """
        return self._bank_accounts

    @bank_accounts.setter
    def bank_accounts(self, bank_accounts):
        """Sets the bank_accounts of this PaymentMethod.


        :param bank_accounts: The bank_accounts of this PaymentMethod.  # noqa: E501
        :type: list[BankAccount]
        """

        self._bank_accounts = bank_accounts



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class SuperMembership(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "super_membership_id": "str",
        "super_fund_id": "str",
        "employee_number": "str",
    }

    attribute_map = {
        "super_membership_id": "SuperMembershipID",
        "super_fund_id": "SuperFundID",
        "employee_number": "EmployeeNumber",
    }

    def __init__(
        self, super_membership_id=None, super_fund_id=None, employee_number=None
    ):  # noqa: E501
        """SuperMembership - a model defined in OpenAPI"""  # noqa: E501

        self._super_membership_id = None
        self._super_fund_id = None
        self._employee_number = None
        self.discriminator = None

        if super_membership_id is not None:
            self.super_membership_id = super_membership_id
        self.super_fund_id = super_fund_id
        self.employee_number = employee_number

    @property
    def super_membership_id(self):
        """Gets the super_membership_id of this SuperMembership.  # noqa: E501

        Xero unique identifier for Super membership  # noqa: E501

        :return: The super_membership_id of this SuperMembership.  # noqa: E501
        :rtype: str
        """
        return self._super_membership_id

    @super_membership_id.setter
    def super_membership_id(self, super_membership_id):
        """Sets the super_membership_id of this SuperMembership.

        Xero unique identifier for Super membership  # noqa: E501

        :param super_membership_id: The super_membership_id of this SuperMembership.  # noqa: E501
        :type: str
        """

        self._super_membership_id = super_membership_id

    @property
    def super_fund_id(self):
        """Gets the super_fund_id of this SuperMembership.  # noqa: E501

        Xero identifier for super fund  # noqa: E501

        :return: The super_fund_id of this SuperMembership.  # noqa: E501
        :rtype: str
        """
        return self._super_fund_id

    @super_fund_id.setter
    def super_fund_id(self, super_fund_id):
        """Sets the super_fund_id of this SuperMembership.

        Xero identifier for super fund  # noqa: E501

        :param super_fund_id: The super_fund_id of this SuperMembership.  # noqa: E501
        :type: str
        """
        if super_fund_id is None:
            raise ValueError(
                "Invalid value for `super_fund_id`, must not be `None`"
            )  # noqa: E501

        self._super_fund_id = super_fund_id

    @property
    def employee_number(self):
        """Gets the employee_number of this SuperMembership.  # noqa: E501

        The membership number assigned to the employee by the super fund.  # noqa: E501

        :return: The employee_number of this SuperMembership.  # noqa: E501
        :rtype: str
        """
        return self._employee_number

    @employee_number.setter
    def employee_number(self, employee_number):
        """Sets the employee_number of this SuperMembership.

        The membership number assigned to the employee by the super fund.  # noqa: E501

        :param employee_number: The employee_number of this SuperMembership.  # noqa: E501
        :type: str
        """
        if employee_number is None:
            raise ValueError(
                "Invalid value for `employee_number`, must not be `None`"
            )  # noqa: E501

        self._employee_number = employee_number



# -*- coding: utf-8 -*-
import pytest

from xero_python.single_dispatch_str import single_dispatch_str


def test_single_dispatch_str():
    @single_dispatch_str
    def decoder(key, data):
        raise ValueError("can't decode {!r} no decoder found".format(key))

    @decoder.register("Connection")
    def _(key, data):
        return key

    def org_decoder(key, data):
        return data

    decoder.register("Organisation", org_decoder)

    @decoder.register("Identity")
    @decoder.register("User")
    @decoder.register("Person")
    def _(key, data):
        return key + data

    value = decoder("Connection", "1")
    assert value == "Connection"
    value = decoder("Organisation", "2")
    assert value == "2"
    with pytest.raises(ValueError):
        decoder("Account", "0")

    assert decoder("Identity", "") == "Identity"
    assert decoder("User", "2") == "User2"
    assert decoder("Person", " 3") == "Person 3"


def test_single_dispatch_str_empty_constructor():
    @single_dispatch_str()
    def parse(key, data):
        raise ValueError("can't parse {!r} no parser found".format(key))

    @parse.register("Connection")
    def _(key, data):
        return key

    def org_parser(key, data):
        return data

    parse.register("Organisation", org_parser)

    @parse.register("Identity")
    @parse.register("User")
    @parse.register("Person")
    def _(key, data):
        return key + data

    assert parse("Connection", "1") == "Connection"
    assert parse("Organisation", "2") == "2"
    with pytest.raises(ValueError):
        parse("Account", "0")

    assert parse("Identity", "") == "Identity"
    assert parse("User", "2") == "User2"
    assert parse("Person", " 3") == "Person 3"


def test_single_dispatch_str_key():
    def key_case_insensitive(key, data):
        return key.lower()

    @single_dispatch_str(key=key_case_insensitive)
    def saver(key, data):
        raise ValueError("can't save {!r} no saver found".format(key))

    @saver.register("connection")
    def _(key, data):
        return key

    def org_saver(key, data):
        return data

    saver.register("organisation", org_saver)

    @saver.register("identity")
    @saver.register("user")
    @saver.register("person")
    def _(key, data):
        return key + data

    assert saver("Connection", "1") == "Connection"
    assert saver("Organisation", "2") == "2"
    with pytest.raises(ValueError):
        saver("Account", "0")

    assert saver("Identity", "") == "Identity"
    assert saver("User", "2") == "User2"
    assert saver("Person", " 3") == "Person 3"



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class ExpenseClaim(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "expense_claim_id": "str",
        "status": "str",
        "payments": "list[Payment]",
        "user": "User",
        "receipts": "list[Receipt]",
        "updated_date_utc": "datetime[ms-format]",
        "total": "float",
        "amount_due": "float",
        "amount_paid": "float",
        "payment_due_date": "date[ms-format]",
        "reporting_date": "date[ms-format]",
        "receipt_id": "str",
    }

    attribute_map = {
        "expense_claim_id": "ExpenseClaimID",
        "status": "Status",
        "payments": "Payments",
        "user": "User",
        "receipts": "Receipts",
        "updated_date_utc": "UpdatedDateUTC",
        "total": "Total",
        "amount_due": "AmountDue",
        "amount_paid": "AmountPaid",
        "payment_due_date": "PaymentDueDate",
        "reporting_date": "ReportingDate",
        "receipt_id": "ReceiptID",
    }

    def __init__(
        self,
        expense_claim_id=None,
        status=None,
        payments=None,
        user=None,
        receipts=None,
        updated_date_utc=None,
        total=None,
        amount_due=None,
        amount_paid=None,
        payment_due_date=None,
        reporting_date=None,
        receipt_id=None,
    ):  # noqa: E501
        """ExpenseClaim - a model defined in OpenAPI"""  # noqa: E501

        self._expense_claim_id = None
        self._status = None
        self._payments = None
        self._user = None
        self._receipts = None
        self._updated_date_utc = None
        self._total = None
        self._amount_due = None
        self._amount_paid = None
        self._payment_due_date = None
        self._reporting_date = None
        self._receipt_id = None
        self.discriminator = None

        if expense_claim_id is not None:
            self.expense_claim_id = expense_claim_id
        if status is not None:
            self.status = status
        if payments is not None:
            self.payments = payments
        if user is not None:
            self.user = user
        if receipts is not None:
            self.receipts = receipts
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc
        if total is not None:
            self.total = total
        if amount_due is not None:
            self.amount_due = amount_due
        if amount_paid is not None:
            self.amount_paid = amount_paid
        if payment_due_date is not None:
            self.payment_due_date = payment_due_date
        if reporting_date is not None:
            self.reporting_date = reporting_date
        if receipt_id is not None:
            self.receipt_id = receipt_id

    @property
    def expense_claim_id(self):
        """Gets the expense_claim_id of this ExpenseClaim.  # noqa: E501

        Xero generated unique identifier for an expense claim  # noqa: E501

        :return: The expense_claim_id of this ExpenseClaim.  # noqa: E501
        :rtype: str
        """
        return self._expense_claim_id

    @expense_claim_id.setter
    def expense_claim_id(self, expense_claim_id):
        """Sets the expense_claim_id of this ExpenseClaim.

        Xero generated unique identifier for an expense claim  # noqa: E501

        :param expense_claim_id: The expense_claim_id of this ExpenseClaim.  # noqa: E501
        :type: str
        """

        self._expense_claim_id = expense_claim_id

    @property
    def status(self):
        """Gets the status of this ExpenseClaim.  # noqa: E501

        Current status of an expense claim – see status types  # noqa: E501

        :return: The status of this ExpenseClaim.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ExpenseClaim.

        Current status of an expense claim – see status types  # noqa: E501

        :param status: The status of this ExpenseClaim.  # noqa: E501
        :type: str
        """
        allowed_values = [
            "SUBMITTED",
            "AUTHORISED",
            "PAID",
            "VOIDED",
            "DELETED",
            "None",
        ]  # noqa: E501

        if status:
            if status not in allowed_values:
                raise ValueError(
                    "Invalid value for `status` ({0}), must be one of {1}".format(  # noqa: E501
                        status, allowed_values
                    )
                )

        self._status = status

    @property
    def payments(self):
        """Gets the payments of this ExpenseClaim.  # noqa: E501

        See Payments  # noqa: E501

        :return: The payments of this ExpenseClaim.  # noqa: E501
        :rtype: list[Payment]
        """
        return self._payments

    @payments.setter
    def payments(self, payments):
        """Sets the payments of this ExpenseClaim.

        See Payments  # noqa: E501

        :param payments: The payments of this ExpenseClaim.  # noqa: E501
        :type: list[Payment]
        """

        self._payments = payments

    @property
    def user(self):
        """Gets the user of this ExpenseClaim.  # noqa: E501


        :return: The user of this ExpenseClaim.  # noqa: E501
        :rtype: User
        """
        return self._user

    @user.setter
    def user(self, user):
        """Sets the user of this ExpenseClaim.


        :param user: The user of this ExpenseClaim.  # noqa: E501
        :type: User
        """

        self._user = user

    @property
    def receipts(self):
        """Gets the receipts of this ExpenseClaim.  # noqa: E501


        :return: The receipts of this ExpenseClaim.  # noqa: E501
        :rtype: list[Receipt]
        """
        return self._receipts

    @receipts.setter
    def receipts(self, receipts):
        """Sets the receipts of this ExpenseClaim.


        :param receipts: The receipts of this ExpenseClaim.  # noqa: E501
        :type: list[Receipt]
        """

        self._receipts = receipts

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this ExpenseClaim.  # noqa: E501

        Last modified date UTC format  # noqa: E501

        :return: The updated_date_utc of this ExpenseClaim.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this ExpenseClaim.

        Last modified date UTC format  # noqa: E501

        :param updated_date_utc: The updated_date_utc of this ExpenseClaim.  # noqa: E501
        :type: datetime
        """

        self._updated_date_utc = updated_date_utc

    @property
    def total(self):
        """Gets the total of this ExpenseClaim.  # noqa: E501

        The total of an expense claim being paid  # noqa: E501

        :return: The total of this ExpenseClaim.  # noqa: E501
        :rtype: float
        """
        return self._total

    @total.setter
    def total(self, total):
        """Sets the total of this ExpenseClaim.

        The total of an expense claim being paid  # noqa: E501

        :param total: The total of this ExpenseClaim.  # noqa: E501
        :type: float
        """

        self._total = total

    @property
    def amount_due(self):
        """Gets the amount_due of this ExpenseClaim.  # noqa: E501

        The amount due to be paid for an expense claim  # noqa: E501

        :return: The amount_due of this ExpenseClaim.  # noqa: E501
        :rtype: float
        """
        return self._amount_due

    @amount_due.setter
    def amount_due(self, amount_due):
        """Sets the amount_due of this ExpenseClaim.

        The amount due to be paid for an expense claim  # noqa: E501

        :param amount_due: The amount_due of this ExpenseClaim.  # noqa: E501
        :type: float
        """

        self._amount_due = amount_due

    @property
    def amount_paid(self):
        """Gets the amount_paid of this ExpenseClaim.  # noqa: E501

        The amount still to pay for an expense claim  # noqa: E501

        :return: The amount_paid of this ExpenseClaim.  # noqa: E501
        :rtype: float
        """
        return self._amount_paid

    @amount_paid.setter
    def amount_paid(self, amount_paid):
        """Sets the amount_paid of this ExpenseClaim.

        The amount still to pay for an expense claim  # noqa: E501

        :param amount_paid: The amount_paid of this ExpenseClaim.  # noqa: E501
        :type: float
        """

        self._amount_paid = amount_paid

    @property
    def payment_due_date(self):
        """Gets the payment_due_date of this ExpenseClaim.  # noqa: E501

        The date when the expense claim is due to be paid YYYY-MM-DD  # noqa: E501

        :return: The payment_due_date of this ExpenseClaim.  # noqa: E501
        :rtype: date
        """
        return self._payment_due_date

    @payment_due_date.setter
    def payment_due_date(self, payment_due_date):
        """Sets the payment_due_date of this ExpenseClaim.

        The date when the expense claim is due to be paid YYYY-MM-DD  # noqa: E501

        :param payment_due_date: The payment_due_date of this ExpenseClaim.  # noqa: E501
        :type: date
        """

        self._payment_due_date = payment_due_date

    @property
    def reporting_date(self):
        """Gets the reporting_date of this ExpenseClaim.  # noqa: E501

        The date the expense claim will be reported in Xero YYYY-MM-DD  # noqa: E501

        :return: The reporting_date of this ExpenseClaim.  # noqa: E501
        :rtype: date
        """
        return self._reporting_date

    @reporting_date.setter
    def reporting_date(self, reporting_date):
        """Sets the reporting_date of this ExpenseClaim.

        The date the expense claim will be reported in Xero YYYY-MM-DD  # noqa: E501

        :param reporting_date: The reporting_date of this ExpenseClaim.  # noqa: E501
        :type: date
        """

        self._reporting_date = reporting_date

    @property
    def receipt_id(self):
        """Gets the receipt_id of this ExpenseClaim.  # noqa: E501

        The Xero identifier for the Receipt e.g. e59a2c7f-1306-4078-a0f3-73537afcbba9  # noqa: E501

        :return: The receipt_id of this ExpenseClaim.  # noqa: E501
        :rtype: str
        """
        return self._receipt_id

    @receipt_id.setter
    def receipt_id(self, receipt_id):
        """Sets the receipt_id of this ExpenseClaim.

        The Xero identifier for the Receipt e.g. e59a2c7f-1306-4078-a0f3-73537afcbba9  # noqa: E501

        :param receipt_id: The receipt_id of this ExpenseClaim.  # noqa: E501
        :type: str
        """

        self._receipt_id = receipt_id



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class TrackingCategories(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"tracking_categories": "list[TrackingCategory]"}

    attribute_map = {"tracking_categories": "TrackingCategories"}

    def __init__(self, tracking_categories=None):  # noqa: E501
        """TrackingCategories - a model defined in OpenAPI"""  # noqa: E501

        self._tracking_categories = None
        self.discriminator = None

        if tracking_categories is not None:
            self.tracking_categories = tracking_categories

    @property
    def tracking_categories(self):
        """Gets the tracking_categories of this TrackingCategories.  # noqa: E501


        :return: The tracking_categories of this TrackingCategories.  # noqa: E501
        :rtype: list[TrackingCategory]
        """
        return self._tracking_categories

    @tracking_categories.setter
    def tracking_categories(self, tracking_categories):
        """Sets the tracking_categories of this TrackingCategories.


        :param tracking_categories: The tracking_categories of this TrackingCategories.  # noqa: E501
        :type: list[TrackingCategory]
        """

        self._tracking_categories = tracking_categories
